<?php

namespace App\Controllers;

use App\Models\AdminModel;
use App\Models\AdminPermissionModel;
use App\Models\AdminRoleModel;
use App\Models\AdminModuleModel;
use App\Models\AdminRoleModuleModel;

class Admin extends BaseController
{
	public function getAll()
	{

		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.user.admin.main'),
			'menu' => 'admin',
		];

		return view('admin/index', $data);

	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$adminModel = new AdminModel();

		$adminData = $adminModel->getAll(); 

		$data = [];



		if (@$adminData) {
        	foreach ($adminData as $index => $val ) {

        	$btn_delete = '';
        	$btn_permission = '';
        	
        	if ($val->id != 1) {

        	$btn_permission = '<a href="'.base_url().'/admin/permission/create/'. $val->id .'" class="btn btn-warning btn-icon" '.lang('Action.tooltip.set_permission').'>'.lang('Action.icon.lg.set_permission').'</a>';
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/admin/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';
        	}

            $btn_edit = '&nbsp;<a href="'.base_url().'/admin/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['email'] = $val->email;
            $value['fullname'] = $val->fullname;
            $value['phone'] = $val->phone;
            $value['insert_at'] = $val->insert_at;
            $value['last_login'] = $val->last_login;
            $value['action'] = $btn_permission.$btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addAdmin()
	{
		if (!$this->request->isAjax()) {
			$adminRoleModel = new AdminRoleModel();

			$dataAdminRole = $adminRoleModel->getAll();

			$data = [
				'data_role' => (isset($dataAdminRole))? $dataAdminRole : [],
				'title' => 'Admin',
				'menu' => 'admin',
			];

			return view('admin/create', $data);
		}

		if (!$this->validate([
            'fullname'          => ['label' => 'Nama Lengkap', 'rules' => 'required|min_length[3]|max_length[255]'],
            'email'             => ['label' => 'Email', 'rules' => 'required|max_length[255]|valid_email|is_unique[admins.admin_email]'],
            'phone'             => ['label' => 'No HP', 'rules' => 'required|numeric|max_length[255]|is_unique[admins.admin_phone]' ],
            'role_id.0'             => ['label' => 'Role', 'rules' => 'required' ],
            'password'          => ['label' => 'Password', 'rules' => 'required|min_length[6]'],
            'allowed_login'     => ['label' => 'Ijinkan Login', 'rules' => 'required|integer|in_list[0,1]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'admin_fullname'=> $this->request->getPost('fullname'),
			'admin_email' 	=> $this->request->getPost('email'),
			'admin_phone' 	=> $this->request->getPost('phone'),
            'admin_password'=> hashingPassword($this->request->getPost('password')),
			'admin_allowed_login' => $this->request->getPost('allowed_login'),
            'admin_insert_admin_id' => $this->session->get('id'),
            'admin_active'          => 1
        ];

		$adminModel = new AdminModel();
		$adminPermissionModel = new AdminPermissionModel();

		$adminData = $adminModel->addAdmin($data); 
		

		if($adminData){

			$roleID = $this->request->getPost('role_id');
			foreach ($roleID as $key => $value) {
	      		$data2 = [
					'ap_admin_id' => $adminData,
					'ap_role_id' => $value,
					'ap_insert_admin_id' => $this->session->get('id'),
					'ap_active' => 1
				];

				$adminPermissionData = $adminPermissionModel->addAdminPermission($data2);
			}

		};

		if ($adminData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'admin'
			];

			return json_encode($data);
		}

	}

	public function updateAdmin($id)
	{
		if (!$this->request->isAjax()) {
			$adminModel = new AdminModel();

			$dataAdmin = $adminModel->getByID($id);

			$data = [
				'data' => (isset($dataAdmin))? $dataAdmin : [],
				'title' => lang('Menu.user.admin.main'),
				'menu' => 'admin',
			];

			return view('admin/edit',  $data);
		}

		if (!$this->validate([
            'fullname'          => ['label' => 'Nama lengkap', 'rules' => 'required|min_length[3]|max_length[255]'],
            'phone'             => ['label' => 'No HP', 'rules' => 'required|numeric|max_length[255]' ],
            'allowed_login'     => ['label' => 'Ijinkan login', 'rules' => 'required|integer|in_list[0,1]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'admin_fullname'=> $this->request->getPost('fullname'),
			'admin_phone' 	=> $this->request->getPost('phone'),
			'admin_allowed_login' => $this->request->getPost('allowed_login'),
            'admin_update_admin_id' => $this->session->get('id'),
            'admin_active'          => 1
        ];

        $password = $this->request->getPost('password');

        if (!empty($password)){
            $data['admin_password'] = hashingPassword($password);
        }

		$adminModel = new AdminModel();
		$adminPermissionModel = new AdminPermissionModel();

		$adminData = $adminModel->updateAdmin($data, $id); 

		if ($adminData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'admin'
			];

			return json_encode($data);
		}
	}

	public function deleteAdmin($id)
	{

		$adminModel = new AdminModel();

		$adminData = $adminModel->deleteAdmin($id, $this->session->get('id')); 

		if ($adminData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}


	public function editPassword()
	{
		
		$adminModel = new AdminModel();

		if (!$this->request->isAjax()) {
			

			$id = session()->get('id');
			$dataAdmin = $adminModel->getByID($id);

			$data = [
				'data' => (isset($dataAdmin))? $dataAdmin : [],
				'title' => lang('Menu.user.edit-password'),
				'menu' => 'admin',
			];

			return view('admin/edit-password',  $data);
		}

		$passwordOld = trim($this->request->getVar('password_old'));
		$password = trim($this->request->getVar('password'));
		$passwordConfirmation = trim($this->request->getVar('password_confirmation'));

		if (!$this->validate([
            'password_old'          => ['label' => 'Password', 'rules' => 'required|max_length[255]'],
            'password'          => ['label' => 'Password', 'rules' => 'required|min_length[5]|max_length[255]'],
            'password_confirmation' => ['label' => 'Ulangi Password', 'rules' => 'required|max_length[255]' ],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		if($passwordConfirmation != $password){
			$msg = [
                'error' => lang('ErrorMessages.passwordConfirmationNotSame'),
            ];
            return json_encode($msg);
		}

		$id = session()->get('id');
		$dataAdmin = $adminModel->getPasswordByID($id);

		if (!password_verify($passwordOld, $dataAdmin->password)) {
			$msg = [
                'error' => lang('ErrorMessages.passwordOldWrong'),
            ];
            return json_encode($msg);
		}

		$data = [
			'admin_password' 		=> hashingPassword($password),
            'admin_update_admin_id' => $this->session->get('id'),
            'admin_active'          => 1
        ];

		$adminModel = new AdminModel();
		$adminData = $adminModel->updateAdmin($data, $this->session->get('id'));

		if ($adminData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'dashboard'
			];

			return json_encode($data);
		}else{
			$data = [
				'success'=> lang('Validation.error_edit'),
			];

			return json_encode($data);
		}
	}


}
