<?php

namespace App\Controllers;

use App\Models\AdminModel;
use App\Models\AdminPermissionModel;
use App\Models\AdminRoleModel;
use App\Models\AdminModuleModel;
use App\Models\AdminRoleModuleModel;

class AdminLogin extends BaseController
{

    public function __construct(){
        parent::__construct();
    }

	public function index()
	{
		return view('auth/login_view');
	}

	public function login_check(){

		if (!$this->request->isAjax()) {
			return false;
		}

		$adminEmail        = $this->request->getPost('email', FILTER_SANITIZE_STRING);        
        $adminPassword     = $this->request->getPost('password', FILTER_SANITIZE_STRING);        

        $this->validation->setRules([
            'email'     => ['label' => 'Email', 'rules' => 'required|valid_email'],
            'password'  => ['label' => 'Password', 'rules' => 'required']
        ]);

        if (!$this->validation->withRequest($this->request)->run()){
            $errors = $this->validation->getErrors();
            $msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
        } else {
            $adminModel = new AdminModel();    
            $adminPermissionModel = new AdminPermissionModel();    
            $adminRoleModuleModel = new AdminRoleModuleModel();    

            $adminData = $adminModel->checkLogin($adminEmail, $adminPassword);

            if($adminData){

            	$adminPermissionData = $adminPermissionModel->getByAdminID($adminData->id); 

				$slugRole = [];
				if (isset($adminPermissionData)) {
					foreach ($adminPermissionData as $key => $value) {
						$slugRole[] = $value->module_slug;
					}
				}

				$ses_data = [
                    'id' => $adminData->id,
                    'fullname' => $adminData->fullname,
                    'email' => $adminData->email,
                    'phone' => $adminData->phone,
                    'session_admin_token' => 1,
                    'array_slug_role_module' => $slugRole
                ];

                $this->session->set($ses_data);
                
				// set_cookie([
				// 	'name'   	=> 'refreshToken',
				// 	'value'  	=> $data->refresh_token,
				// 	'expire' 	=> $data->refresh_token_expired_at,
				// 	'httpOnly'	=> true
				// ]);
                     
				$msg = [
	                'success' =>  lang('Validation.success_login'),
	                'redirect' => 'dashboard',
	            ];

	            return json_encode($msg);
            }else{
                $msg = [
	                'error' => lang('Validation.error_login'),
	            ];
	            return json_encode($msg);
            }
        }

	}

	public function logout()
    {
        $this->session->destroy();
        return redirect()->to('/');
    }

    public function request_forgot_password()
    {
    	$data = [
    		'title' => 'Forgot Password',
    	];

    	return view('auth/forgot-password', $data);
    }

    public function request_forgot_password_send()
    {
    	if (!$this->request->isAjax()) {
			return false;
		}

		$email = $this->request->getPost('email');

		$data = [
			'email' => $email,
		];
		
		$result = json_decode(sendRequest($this->api['endpoint'].'/request_forgot_password', 'POST', $this->api['headers'], $data));

		if (!$result) {
			return false;
		} else {

			if ($result->status == 400) {

	            $errors = $result->data->error_message;

				$msg = [
	                'error' => errorMessage($errors),
	            ];

	           return json_encode($msg);
	        } else {
                     
				$msg = [
	                'success' =>  lang('Validation.success_request_reset_password'),
	            ];

	            return json_encode($msg);
			}
		}
    }


    public function reset_password($token)
    {
    	$data = [
    		'title' => 'Reset Password',
    	];
    	return view('auth/reset-password', $data);
    }

    public function reset_password_send($token)
    {
    	if (!$this->request->isAjax()) {
			return false;
		}

		$password = $this->request->getPost('password');
		$password_confirmation = $this->request->getPost('password_confirmation');

		$data = [
			'password' => $password,
			'password_confirmation' => $password_confirmation
		];
		
		$result = json_decode(sendRequest($this->api['endpoint'].'/forgot_password/'.$token, 'POST', $this->api['headers'], $data));

		if (!$result) {
			return false;
		} else {

			if ($result->status == 400) {

	            $errors = $result->data->error_message;

				$msg = [
	                'error' => errorMessage($errors),
	            ];

	           return json_encode($msg);
	        } else {
                     
				$msg = [
	                'success' =>  lang('Validation.success_reset_password'),
	            ];

	            return json_encode($msg);
			}
		}
    }
}