<?php
namespace App\Controllers;

use App\Models\ItemModel;
use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\WorkflowModel;
use App\Models\BatchModel;
use App\Models\BatchItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Libraries\Barcode;
use App\Models\AdminRoleModuleModel;
use App\Models\TreeModel;
use App\Models\PrintHistoryModel;
use App\Models\GoldContentModel;
use App\Models\ItemMaterialModel;
use App\Models\BatchItemRequestModel;
use App\Models\BatchItemTaskModel;

class Batch extends BaseController
{
	public function getAll()
	{
		
		if($this->request->isAjax()){

			if (@$this->request->getGet('act') == 'orderItemData') {
				return $this->orderItemData();
			}else if(@$this->request->getGet('act') == 'treeData'){
				return $this->treeData();
			}else if(@$this->request->getGet('act') == 'orderItemByCheck') {
				return $this->orderItemByCheck();
			}
			
			return $this->data();
		}

		$treeModel = new TreeModel();
		$goldContentModel = new GoldContentModel();
		
		$data = [
			'title' => lang('Menu.order.batch.main'),
			'menu' => 'batch',
			'data_gc' => $goldContentModel->getAll(),
		    'data_tree' => $treeModel->getAll(),
		];

		return view('batch/index', $data);
	}

	public function treeByID(){
		$treeID = $this->request->getVar('tree_id');

		$treeModel = new TreeModel();

		$treeData = $treeModel->getByID($treeID); 

		return json_encode($treeData);
	}

	public function gcByID(){
		$gcID = $this->request->getVar('gc_id');

		$gcModel = new GoldContentModel();

		$gcData = $gcModel->getByID($gcID); 

		return json_encode($gcData);
	}

	public function data()
	{
		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$paramType = $this->request->getGet('paramType');

		if ($paramType == 'treeByID') {
			return $this->treeByID();
		}else if($paramType == 'gcByID'){
			return $this->gcByID();
		}

		$fromDate = trim($this->request->getVar('from_date'));
		$toDate = trim($this->request->getVar('to_date'));
		$note = trim($this->request->getVar('note'));
		$gcID = trim($this->request->getVar('gc_id'));
		$treeID = trim($this->request->getVar('tree_id'));
		$status = trim($this->request->getVar('status'));
		
		$batchModel = new BatchModel();

		$paramWhere = ' AND batch_status IN ("Proses", "New", "Cancelled") AND batch_active=1 ';
		$paramJoin = '';

		if ($fromDate) {
			$paramWhere .= ' AND DATE(batch_start_date) >= "'.$fromDate.'" ';
		}

		if ($toDate) {
			$paramWhere .= ' AND DATE(batch_start_date) <= "'.$toDate.'" ';
		}

		if ($note) {
			$paramWhere .= ' AND ( batch_code LIKE "%'.$note.'%" OR (SELECT COUNT(*) FROM batch_items 
			                    JOIN order_items ON order_items.oi_id=batch_items.bi_oi_id
			                    JOIN orders ON order_items.oi_order_id=orders.order_id
								WHERE batch_items.bi_batch_id=batches.batch_id AND orders.order_note LIKE "%'.$note.'%" ) > 0
							) ';
		}

		if ($gcID) {
			$paramWhere .= ' AND gold_contents.gc_id="'.$gcID.'" ';
		}

		if ($treeID) {
			$paramWhere .= ' AND trees.tree_id="'.$treeID.'" ';
		}

		if ($status) {
			$paramWhere .= ' AND batch_status = "'.$status.'" ';
		}

		$batchData = $batchModel->getAll([
			'where'=> $paramWhere,
			'orderby'=> 'batches.batch_insert_at ASC',
		]);

		$data = [];

		if (@$batchData) {
        	foreach ($batchData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/batch/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_edit = '&nbsp;<a href="'.base_url().'/batch/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

	            $btn_print = '&nbsp;<a href="'.base_url().'/batch/print/'.$val->id.'" target="_blank" class="btn btn-icon btn-secondary" '.lang('Action.tooltip.report').' >'.lang('Action.icon.lg.report').'</a>&nbsp;
	            	<a href="'.base_url().'/batch/note/'.$val->id.'" class="btn btn-warning btn-icon" target="_blank" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';

	            $btn_start = '';

	            if ($val->status != 'Proses') {
	            	$btn_start = '&nbsp;<a href="javascript::void()" onclick="startData('.$val->id.')" class="btn btn-primary btn-icon button'.$val->id.' " '.lang('Action.tooltip.start').'>'.lang('Action.icon.lg.start').'</a>';

	            }

	            if ($val->status == 'Completed' || $val->status == 'Proses') {
	            	$btn_edit = '';
	            	$btn_delete = '';
	            	$btn_print .= '&nbsp;<a href="'.base_url().'/batch/item-task/'.$val->id.'" class="btn btn-dark btn-icon" target="_blank" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';
	            }

	            $value['code'] = $val->code;
	            $value['tree_name'] = $val->tree_name;
	            $value['gold_content'] = $val->gc_name;
	            $value['start_date'] = $val->start_date;
	            $value['due_date'] = $val->due_date;
	            $value['status'] = $val->status;
	            if ($value['status'] == 'Cancelled') {
	            	$value['action'] = '<div class="cancelled">'.$val->remark.'<div>';
	            }else{
	            	$value['action'] = $btn_start.$btn_print.$btn_edit.$btn_delete;
	            }

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addBatch()
	{
		if (!$this->request->isAjax()) {
			$workflowModel = new WorkflowModel();

			$treeModel = new TreeModel();
			$goldContentModel = new GoldContentModel();

			$orderItemModel = new OrderItemModel();

			$batchItemModel = new BatchItemModel();

			// $arrayOrderItem = [];
			// $batchItemData = $batchItemModel->getAll();

			// foreach ($batchItemData as $key => $value) {
			// 	$arrayOrderItem[] = $value->order_item_id;
			// }

			$data = [
				'title' => lang('Menu.order.batch.main'),
				'data_workflow' => $workflowModel->getAll(),
				'data_order_item' => $orderItemModel->getAll(),
				// 'array_order_item' => $arrayOrderItem,
				'data_gc' => $goldContentModel->getAll(),
				'data_tree' => $treeModel->getAll(),
				'menu' => 'batch',
			];

			return view('batch/create', $data);
		}else{

			$rules = [
	            'start_date'        => ['label' => 'Start date', 'rules' => 'required|max_length[255]'],
	            'due_date'          => ['label' => 'Due date', 'rules' => 'required|max_length[255]'],
	            'tree_id'           => ['label' => 'Pohon', 'rules' => 'required|max_length[255]'],
	            'gc_id'             => ['label' => 'Kadar Emas', 'rules' => 'required|max_length[255]'],
	            'order_item_id.0'   => ['label' => 'Order Item', 'rules' => 'required'],
	        ];

			if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$batchModel = new BatchModel();
			$batchItemModel = new BatchItemModel();
			$orderItemModel = new OrderItemModel();
			$itemModel = new ItemModel();

			$data = [
				'batch_code'		    => $this->batchCode(date('Y-m-d')),
				'batch_workflow_id' 	=> $this->request->getPost('workflow_id'),
				'batch_gc_id' 			=> $this->request->getPost('gc_id'), 
				'batch_tree_id' 		=> $this->request->getPost('tree_id'),
				'batch_start_date' 		=> $this->request->getPost('start_date'),
				'batch_due_date' 		=> $this->request->getPost('due_date'),
				'batch_status' 			=> 'New',
				'batch_insert_admin_id' => $this->session->get('id'),
	            'batch_active'          => 1
	        ];

	        $orderItemId = $this->request->getPost('order_item_id');
	        $treeId = $this->request->getPost('tree_id');
	        $gcID = $this->request->getPost('gc_id');

	        $gcModel = new GoldContentModel();
	        $gcData = $gcModel->getByID($gcID);

	        $gcMultiplier = $gcData->multiplier;

	        $treeID = $this->request->getPost('tree_id');
	        $stoneQuantity = $this->request->getPost('stone_quantity');

	   		$treeModel = new TreeModel();
	        $treeData = $treeModel->getByID($treeID);
	        $treeWeight = $treeData->weight;

	        $oiStartDate = $this->request->getPost('oi_start_date');

	        $sku = $this->request->getPost('sku');

	        $key = 0;
	        $dataPath = '';

	        $s3Library = new S3Library();

			if($orderItemId != null){

				$implodeData = implode(',', $orderItemId);

				$resultOrderItem = $orderItemModel->getAll(['where' => ' AND oi_id IN ('.$implodeData.') ']);

				$itemMaterialModel = new ItemMaterialModel();

				$dataStone = [];

				foreach ($resultOrderItem as $key => $value) {
					$param['where'] = ' AND im_item_id="'.$value->item_id.'" AND im_active=1 ';
					$itemMaterialData = $itemMaterialModel->getAll($param); 
					if (!empty($itemMaterialData)) {
						foreach ($itemMaterialData as $key1 => $value1) {
							if ($value1->source == 'Gudang Batu') {	
								if (!array_key_exists("material_bulk[".$value1->name."]", $dataStone)) {
									$dataStone["material_bulk[".$value1->name."]"] = 0;
								}
								
								$dataStone["material_bulk[".$value1->name."]"] += intval($value1->quantity)*$value->quantity;
							}
						}
					}
				}
				if (!empty($dataStone)) {
					$dataStone["user_id"] = 1;
					$dataStone["description"] = $data['batch_code'];
				}

				if (!empty($dataStone)) {
					$api['headers'] = [
						'Accept'        => 'application/json',
						'Authorization: Bearer '.GUDANG_BATU_TOKEN
					];

					$resultMaterialOut = json_decode(sendRequest(GUDANG_BATU_URL_VALIDATE, 'POST', $api['headers'], $dataStone ));

					if ($resultMaterialOut->status != 200) {
						// return result_message_json($resultMaterialOut, 'add', 'gudang_batu');
						// $data = [
						// 	'error'=> lang('Validation.error_add'),
						// ];

						$data = [
							'error'=> errorMessage($resultMaterialOut->data->error_message),
						];

						return json_encode($data);
					}
				}
			}

			
			$batchID = $batchModel->addBatch($data);
			
			$adminRoleModuleModel = new AdminRoleModuleModel();

			$query = [
				'section' => 'Batch',
				'groupby' => 'admin_phone'
			];
	        $adminRoleModuleData = $adminRoleModuleModel->getAllWithModule($query);

		    //     foreach ($adminRoleModuleData as $key => $value) {
					// $result = sendWhatsApp($value->admin_phone, 'New Batch | Code Batch : '.$data['batch_code']);
		    //     }

			$dataBi = [];

			if($orderItemId != null){

				foreach ($orderItemId as $key_ => $value_) {

					$resultOrderItem = $orderItemModel->getByID($value_);

					$dataBi[] = [
						'bi_batch_id' => $batchID,
						'bi_oi_id' => $value_,
						'bi_tree_id' => $treeId,
						'bi_tree_weight' => $treeWeight,
						// 'bi_total_weight' => $treeTotalWeight,
						// 'bi_stone_type' => $resultOrderItem->item_stone_type,
						'bi_stone_quantity' => @$stoneQuantity[$value_],
						// 'bi_stone_weight' => @$stoneWeight[$value_],
						// 'bi_gold_weight' => $goldWeight,
			            'bi_insert_admin_id' => $this->session->get('id'),
			            'bi_active'          => 1
					];

					$dataOi = [
						'oi_start_date' => ($resultOrderItem->start_date == '')? $data['batch_start_date'] : $resultOrderItem->start_date,
			            'oi_update_admin_id' => $this->session->get('id'),
					];

					$orderItemDataByID = $orderItemModel->getByID($orderItemId[$key]);

					$dataItem = [
						'item_workflow_id' => ($orderItemDataByID->workflow_id == '')? $data['batch_workflow_id'] : $orderItemDataByID->workflow_id,
			            'item_update_admin_id' => $this->session->get('id'),
					];

					$itemData = $itemModel->updateItem($dataItem, $orderItemDataByID->item_id);

					$orderItemData = $orderItemModel->updateOrderItem($dataOi,  $orderItemId[$key]);
				}

			}

			$batchItemData = $batchItemModel->addBatchItemBulk($dataBi);


			$batchItemResult = $batchItemModel->getAll(['where' => ' AND batch_id="'.$batchID.'" ']);

			$dataBatchItemRequest = [];

			foreach ($batchItemResult as $key => $value) {
				$param['where'] = ' AND im_item_id="'.$value->item_id.'" AND im_active=1 ';
				$itemMaterialData = $itemMaterialModel->getAll($param); 

				if (!empty($itemMaterialData)) {
					foreach ($itemMaterialData as $key1 => $value1) {
						if ($value1->source == 'Gudang Batu') {
							$dataBatchItemRequest[] = [
								'bir_bi_id' => $value->id,
								'bir_item_source' => $value1->source,
								'bir_item_sku' => $value1->name,
								'bir_item_name' => $value1->name,
								'bir_item_quantity' => (intval($value1->quantity)*$value->quantity),
								'bir_item_weight' => $value->stone_weight,
								'bir_status' => 'New',
								'bir_insert_at' => getCurrentTime(),
								'bir_insert_admin_id' => $this->session->get('id'),
								'bir_active' => 1
							];
						}
					}
				}
			}

			$batchItemRequestModel = new BatchItemRequestModel();

			if ($dataBatchItemRequest) {
				$batchItemRequestAdd = $batchItemRequestModel->addBatchItemRequestBulk($dataBatchItemRequest);
			}


			if($orderItemId != null){

				$implodeData = implode(',', $orderItemId);

				$resultOrderItem = $orderItemModel->getAll(['where' => ' AND oi_id IN ('.$implodeData.') ']);

				$itemMaterialModel = new ItemMaterialModel();

				$dataStone = [];

				foreach ($resultOrderItem as $key => $value) {
					$param['where'] = ' AND im_item_id="'.$value->item_id.'" AND im_active=1 ';
					$itemMaterialData = $itemMaterialModel->getAll($param); 
					if (!empty($itemMaterialData)) {
						foreach ($itemMaterialData as $key1 => $value1) {
							if ($value1->source == 'Gudang Batu') {	
								if (!array_key_exists("material_bulk[".$value1->name."]", $dataStone)) {
									$dataStone["material_bulk[".$value1->name."]"] = 0;
								}
								
								$dataStone["material_bulk[".$value1->name."]"] += intval($value1->quantity)*$value->quantity;
							}
						}
					}
				}
				if (!empty($dataStone)) {
					$dataStone["user_id"] = 1;
					$dataStone["description"] = $data['batch_code'];
				}

				if (!empty($dataStone)) {
					$api['headers'] = [
						'Accept'        => 'application/json',
						'Authorization: Bearer '.GUDANG_BATU_TOKEN
					];

					$resultMaterialOut = json_decode(sendRequest(GUDANG_BATU_URL_CREATE, 'POST', $api['headers'], $dataStone ));

					if ($resultMaterialOut->status != 200) {
						// return result_message_json($resultMaterialOut, 'add', 'gudang_batu');
						// $data = [
						// 	'error'=> lang('Validation.error_add'),
						// ];

						$data = [
							'error'=> errorMessage($resultMaterialOut->data->error_message),
						];

						return json_encode($data);
					}
				}
			}

			sendWhatsApp('088232340119', 'New Batch | Code Batch : '.$data['batch_code']);
	        sendWhatsApp('08122680110', 'New Batch | Code Batch : '.$data['batch_code']);
	        sendWhatsApp('081390548982', 'New Batch | Code Batch : '.$data['batch_code']);


			if ($batchID && $batchItemData) {
				$data = [
					'success'=> lang('Validation.success_add'),
					'redirect'=> 'batch'
				];

				return json_encode($data);
			}

		}

	}

	public function updateBatch($id)
	{

		$batchModel = new BatchModel();

		$batchData = $batchModel->getByID($id);

		$batchItemModel = new BatchItemModel();

		$batchItemData = $batchItemModel->getByBatchID($id);

		if (!$this->request->isAjax()) {
			if($batchData->status == 'Completed' || $batchData->status == 'Proses' || $batchData->status == 'Deleted' || $batchData->status == 'Cancelled' ){
				return redirect()->to('batch');
			}

		}else{


			if($batchData->status == 'Completed' || $batchData->status == 'Proses' || $batchData->status == 'Deleted' || $batchData->status == 'Cancelled' ){
				$data = [
					'redirect'=> 'batch'
				];
				return json_encode($data);
			}
		}

		$workflowModel = new WorkflowModel();
		if (!$this->request->isAjax()) {

			$workflowModel = new WorkflowModel();
			$treeModel = new TreeModel();
			$goldContentModel = new GoldContentModel();

			$data = [
				'data' => (isset($batchData))? $batchData : [],
				'data_workflow' => $workflowModel->getAll(),
				'data_batch_item' => $batchItemData,
				'data_tree' => $treeModel->getAll(),
				'title' => lang('Menu.order.batch.main'),
				'menu' => 'batch',
				'batchID' => $id,
			];

			return view('batch/edit',  $data);

		}else{

			 $batchItemId = $this->request->getPost('batch_item_id');

	        $treeId = $this->request->getPost('tree_id');
	        $gcID = $this->request->getPost('gc_id');
	        $erpBarcode = $this->request->getPost('erp_barcode');

	        $gcModel = new GoldContentModel();
	        $gcData = $gcModel->getByID($gcID);

	        $gcMultiplier = $gcData->multiplier;

	        $treeID = $this->request->getPost('tree_id');
	        $stoneQuantity = $this->request->getPost('stone_quantity');

	   		$treeModel = new TreeModel();
	        $treeData = $treeModel->getByID($treeID);
	        $treeWeight = $treeData->weight;
	        $treeTotalWeight = $this->request->getPost('total_weight');
	        $diffTreeWeight = $treeTotalWeight - $treeWeight;
	        $stoneWeight = $this->request->getPost('stone_weight');
	        $totalStoneWeight = array_sum($stoneWeight);
	        $goldWeight = ($diffTreeWeight - $totalStoneWeight)*$gcMultiplier;

	        $oiStartDate = $this->request->getPost('oi_start_date');

	        $sku = $this->request->getPost('sku');

	        $key = 0;
	        $dataPath = '';

	        $s3Library = new S3Library();

	        $batchItemRequestModel = new BatchItemRequestModel();
			

	        $rules = [
	            'start_date'             => ['label' => 'Tanggal mulai', 'rules' => 'required|max_length[255]'],
	            'due_date'             => ['label' => 'Batas waktu/tanggal', 'rules' => 'required|max_length[255]'],
	            'batch_item_id.0'  		 => ['label' => 'Batch Item', 'rules' => 'required'],
	        ];

			if ($batchItemId!= null) {
				foreach ($batchItemId as $key => $value) {
					$rules['stone_quantity.'.$value] = ['label' => 'Jumlah Batu', 'rules' => 'max_length[255]'];

					if (@$stoneQuantity[$value] > 0) {
						$rules['stone_weight.'.$value] = ['label' => 'Berat Batu', 'rules' => 'required|max_length[255]'];
					}else{
						$rules['stone_weight.'.$value] = ['label' => 'Berat Batu', 'rules' => 'max_length[255]'];
					}

					$rules['gold_weight.'.$value] = ['label' => 'Berat Emas', 'rules' => 'required|max_length[255]'];

					$batchItemRequestGet = $batchItemRequestModel->getAll(['where' => 'AND bir_status="New" AND bir_bi_id="'.$value.'" AND bir_active=1 ']);

					if(count($batchItemRequestGet) > 0){
						$msg = [
			                'error' => 'Request batu belum disetujui',
			            ];
			            return json_encode($msg);
					}
				}
	        }

			if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$data = [
				'batch_tree_id' 		=> $this->request->getPost('tree_id'),
				'batch_start_date' 		=> $this->request->getPost('start_date'),
				'batch_due_date' 		=> $this->request->getPost('due_date'),
				'batch_status' 			=> 'New',
				'batch_total_weight'	=> $treeTotalWeight,
				'batch_total_stone_weight' => $totalStoneWeight,
				'batch_total_gold_weight' => $goldWeight,
				'batch_update_admin_id' => $this->session->get('id'),
				'batch_update_at'	 	=> getCurrentTime(),
	            'batch_active'          => 1
	        ];

			$batchID = $batchModel->updateBatches($data, $id);

			$orderItemModel = new OrderItemModel();
			
			$dataBi = [];


			if($batchItemId != null){

				foreach ($batchItemId as $key_ => $value_) {

					$dataBi[] = [
						'bi_id' => $value_,
						'bi_tree_id' => $treeId,
						'bi_tree_weight' => $treeWeight,
						'bi_erp_barcode' => @$erpBarcode[$value_],
						'bi_total_weight' => $treeTotalWeight,
						'bi_stone_weight' => @$stoneWeight[$value_],
						'bi_gold_weight' => $goldWeight,
			            'bi_update_admin_id' => $this->session->get('id'),
			            'bi_update_at' => $this->session->get('id'),
			            'bi_active'          => 1
					];
				}
			}

			$batchItemData = $batchItemModel->updateBatchItemBulk($dataBi, 'bi_id');


			if ($batchItemData) {
				$data = [
					'success'=> lang('Validation.success_edit'),
					'redirect'=> 'batch'
				];

				return json_encode($data);
			}
			
		}
	}


	public function deleteBatch($id)
	{

		$batchModel = new BatchModel();
		// $batchItemTaskModel = new BatchItemTaskModel();


		$batchData = $batchModel->getByID($id);
		$api['headers'] = [
			'Accept'        => 'application/json',
			'Authorization: Bearer '.GUDANG_BATU_TOKEN
		];

		$resultMaterialOut = json_decode(sendRequest(GUDANG_BATU_URL_DELETE, 'POST', $api['headers'], ['code' => $batchData->code] ));

		$batchData = $batchModel->deleteBatch($id, $this->session->get('id'));

		$orderItemData = $batchModel->deleteItemBatchByBatch($id, $this->session->get('id'));

		// $batchItemTaskData = $batchItemTaskModel->deleteBatchItemTaskByBatch($id, $this->session->get('id'));

		// if ($resultMaterialOut->status != 200) {
		// 	$data = [
		// 		'error'=> errorMessage($resultMaterialOut->data->error_message),
		// 	];

		// 	return json_encode($data);
		// }

		if ($batchData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}else{
			$data = [
				'error'=> lang('Validation.error_delete'),
			];

			return json_encode($data);
		}
	}

	public function orderItemData(){

		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$gcID = $this->request->getGet('gc_id');
		$note = $this->request->getGet('note');

		$data = [];

		$orderItemModel = new OrderItemModel();

		$batchItemModel = new BatchItemModel();

		$arrayOrderItem = [];
		$batchItemData = $batchItemModel->getAll(["where" => ' AND bi_active=1 AND batch_status != "Cancelled" ']);
		foreach (@$batchItemData as $key => $val1) {
			$arrayOrderItem[] = $val1->order_item_id;
		}

		$query = '';
		if ($gcID == '') {
			// $orderItemData = $orderItemModel->getAll();
			$query .= ' AND items.item_gc_id IS NULL AND order_active=1 AND oi_active=1 ';
		}else{
			$query .= ' AND items.item_gc_id="'.$gcID.'" AND order_active=1 AND oi_active=1 ';
		}

		if ($note != '') {
			$query .= ' AND orders.order_note LIKE "%'.$note.'%" ';
		}

		$param = [
			'where' => $query,
		];

		$oiID = $this->request->getGet('oi_id');

		$arrayOiID = explode(',', $oiID);

		$orderItemData = $orderItemModel->getAll($param);

		$item = [];
		foreach ($orderItemData as $key_ => $value_) {
			if (!in_array($value_->item_id, $item)) {
				$item[] = $value_->item_id;
			}
		}

		$itemImplode = implode(',', $item);
		
		$itemMaterialArray = [];
		
		if (!empty($itemImplode)) {
			
			$itemMaterialModel = new ItemMaterialModel();
			$param = [
				'select' => 'SUM(im_quantity) as "sum_quantity", im_item_id as "item_id" ',
				'where' => ' AND im_item_id IN ('.$itemImplode.') AND im_active=1 ',
				'groupby' => 'im_item_id',
			];

			$itemMaterialData = $itemMaterialModel->getAll($param);

			foreach ($itemMaterialData as $key1 => $value1) {
				if (array_key_exists($value1->item_id, $itemMaterialArray)) {
					$itemMaterialArray[$value1->item_id] += $value1->sum_quantity;
				}else {
					$itemMaterialArray[$value1->item_id] = $value1->sum_quantity;
				}
			}
		}

		if (@$orderItemData) {
        	foreach ($orderItemData as $index => $val ) {
        		if (!in_array($val->id, $arrayOrderItem)) {

        			$checked = (in_array($val->id, $arrayOiID))? 'checked': '';

		            $value['checkbox'] = '<div class="form-check form-check-custom form-check-solid form-check-sm">
										    <input class="form-check-input order-item-id checked-'.$val->id.'" '.$checked.' value="'.$val->id.'" name="order_item_id[]" type="checkbox" value="" id="flexRadioLg'.$index.'"/>
										</div>';
		            $value['name'] = $val->name.'<br>SKU : '.$val->sku.'<br>Alur Kerja : '.$val->workflow_name.'<br>Kadar Emas : '.$val->gc_name;
		            $value['quantity'] = $val->quantity;
		            $value['stone_quantity'] = @$itemMaterialArray[$val->item_id]*$val->quantity;
		            $value['workflow_name'] = $val->workflow_name;
		            $value['customer_name'] = $val->customer_name;
		            $value['order_note'] = $val->order_note;
		            $value['date'] = 'Tanggal Mulai : '.$val->start_date.'<br>Batas Waktu : '.$val->due_date.'<br>Batas Waktu Cor : '.$val->start_due_date;
            		array_push($data,$value);
        		}
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function orderItemByCheck(){

		$orderItemModel = new OrderItemModel();

		$orderIDImplode = $this->request->getGet('order_item_id');

		if (empty($orderIDImplode)) {
			$data = [
				'error'=> lang('Validation.orderItemRequired'),
			];

			return json_encode($data);
		}

		$param = [
			'where' => ' AND oi_id IN ('.$orderIDImplode.') AND oi_active=1 ',
		];


		$orderItemData = $orderItemModel->getAll($param);

		$item = [];
		foreach ($orderItemData as $key_ => $value_) {
			if (!in_array($value_->item_id, $item)) {
				$item[] = $value_->item_id;
			}
		}

		$itemImplode = implode(',', $item);
		$itemMaterialArray = [];
		
		if (!empty($itemImplode)) {
			
			$itemMaterialModel = new ItemMaterialModel();
			$param = [
				'select' => 'SUM(im_quantity) as "sum_quantity", im_item_id as "item_id" ',
				'where' => ' AND im_item_id IN ('.$itemImplode.') AND im_active=1 ',
				'groupby' => 'im_item_id',
			];

			$itemMaterialData = $itemMaterialModel->getAll($param);

			foreach ($itemMaterialData as $key1 => $value1) {
				if (array_key_exists($value1->item_id, $itemMaterialArray)) {
					$itemMaterialArray[$value1->item_id] += $value1->sum_quantity;
				}else {
					$itemMaterialArray[$value1->item_id] = $value1->sum_quantity;
				}
			}

		}

		$html = '';
		foreach ($orderItemData as $key => $value) {
			$html .= '<tr class="row-item">
						<td>
							<div class="form-check form-check-custom form-check-solid form-check-sm">
							    <input class="form-check-input result-order-item-id" checked value="'.$value->id.'" name="order_item_id[]" type="checkbox" value="" id="flexRadioLg'.$value->id.'"/>
							</div>
						</td>
						<td class="min-w-100px">
						<br>
						SKU : '.$value->sku.'<br>
						Barang : '.$value->name.'<br>
						Pelanggan : '.$value->customer_name.'<br>
						Alur Kerja : '.$value->workflow_name.'<br>
						Jumlah Order: '.$value->quantity.'<br>
						Tanggal Mulai : '.$value->start_date.'<br>
						Tenggat Waktu: '.$value->due_date.'<br>
						</td>
						<td class="min-w-100px">'.$value->order_note.'</td>
						<th class="min-w-100px">'.$value->quantity.'</th>
						<th class="min-w-100px"><input type="hidden" class="sum-quantity stone-quantity" name="stone_quantity['.$value->id.']" value="'.(@$itemMaterialArray[$value->item_id]*$value->quantity).'">'.(@$itemMaterialArray[$value->item_id]*$value->quantity).'</th>
					</tr>';
		}

		return json_encode(['data'=> $html]);
	}

	public function note($id)
	{

		$batchModel = new BatchModel();

		$batchData = $batchModel->getByID($id);

		$batchItemModel = new BatchItemModel();

		$batchItemData = $batchItemModel->getByBatchID($id);

		$workflowModel = new WorkflowModel();


		$batchItemRequestModel = new BatchItemRequestModel();
		$batchItemRequestData = $batchItemRequestModel->getAll(['where'=> ' AND  batch_items.bi_batch_id="'.$id.'" ']);
		$batchItemRequestArray = [];
		foreach ($batchItemRequestData as $key => $value) {
			$batchItemRequestArray[$value->bi_id][] = $value->item_sku;
		}

		if (!$this->request->isAjax()) {
			$data = [
				'data' => (isset($batchData))? $batchData : [],
				'data_workflow' => $workflowModel->getAll(),
				'data_batch_item' => $batchItemData,
				'title' => lang('Menu.order.batch.main'),
				'menu' => 'batch',
				'batch_item_request' => $batchItemRequestArray,
			];
			return view('batch/note',  $data);
		}
	}

	public function print($id)
	{

		$batchModel = new BatchModel();

		$batchData = $batchModel->getByID($id);

		// if ($batchData->is_print != 0){
		// 	return json_encode([
		// 		'error' => lang('Validation.error_access_forbidden'),
		// 		'redirect' => 'auth'
		// 	]);
		// }
		$batchItemModel = new BatchItemModel();
		$workflowModel = new WorkflowModel();
		$printHistoryModel = new PrintHistoryModel();
		$batchItemRequestModel = new BatchItemRequestModel();


		$dataBatch = [
			'batch_is_print' 		=> $batchData->is_print+1,
			'batch_update_admin_id' => $this->session->get('id'),
			'batch_update_at' 		=> getCurrentTime(),
            'batch_active'          => 1
        ];
		$batchID = $batchModel->updateBatches($dataBatch, $id);

		$dataPH = [
			'ph_reference_id' => $id,
			'ph_reference_type' => 'batch',
			'ph_timestamp' => getCurrentTime(),
			'ph_admin_id' => $this->session->get('id')
		];
		$printHistoryModel->addPrintHistory($dataPH);
		
		$batchItemData = $batchItemModel->getByBatchID($id);

		$batchItemRequestData = $batchItemRequestModel->getAll(['where'=> ' AND  batch_items.bi_batch_id="'.$id.'" AND (bir_bit_id IS NULL OR bir_bit_id="") AND bir_active=1 ']);
		$batchItemRequestArray = [];
		foreach ($batchItemRequestData as $key => $value) {
			$batchItemRequestArray[$value->bi_id][] = $value;
		}

		$data = [
			'data' => (isset($batchData))? $batchData : [],
			'data_workflow' => $workflowModel->getAll(),
			'data_batch_item' => $batchItemData,
			'batch_item_request' => $batchItemRequestArray,
			'title' => lang('Menu.order.batch.main'),
			'menu' => 'batch',
		];

		return view('batch/print',  $data);
	}


	public function printTask($id){

		$batchItemModel = new BatchItemModel();
		$batchItemData = $batchItemModel->getByBatchID($id);
		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemRequestModel = new BatchItemRequestModel();
		$barcode = new Barcode();


		$bitGroupNote = $batchItemTaskModel->getAllGroupNote(['where' => ' AND batches.batch_id="'.$id.'" ']);

		$array = [];
		foreach ($bitGroupNote as $key => $value) {
			$groupNote = $value->group_note;

			$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$groupNote.'" ', 'orderby' => ' bit_id ASC ']);

			$biID = @$batchItemTaskData[0]->bi_id;
			$batchItemRequestData = $batchItemRequestModel->getAll(['where' => ' AND bir_bi_id="'.$biID.'" AND bir_active=1 ', 'orderby' => ' bir_id ASC ']);


			$batchItemData = @$batchItemModel->getByID($biID)[0];
			$viewBarcode = $barcode->generate($groupNote, "SVG");

			$array[] = [
				'batch_item_task' => $batchItemTaskData,
				'batch_item' => $batchItemData,
				'batch_item_request' => $batchItemRequestData,
				'barcode' => $viewBarcode,
			];

		}

		$data = [
			'data' => $array
		];

		return view('batch/print-bit', $data);
	}


	public function treeData(){
		$id = $this->request->getVar('tree_id');

		$treeModel = new TreeModel();

		$treeData = $treeModel->getByID($id);

		return json_encode($treeData);
	}

	public function batchCode($date){
    	$batchModel = new BatchModel();
		$batchNota = $batchModel->setCode();

		if ($batchNota) {
			$number = $batchNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'BP/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
    }


}