<?php
namespace App\Controllers;

use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\CustomerModel;
use App\Models\WorkflowModel;
use App\Models\SizeModel;
use App\Models\GoldContentModel;
use App\Models\GoldTypeModel;
use App\Models\CategoryModel;
use App\Models\ItemModel;
use Ramsey\Uuid\Uuid;
use App\Models\BatchItemModel;
use App\Models\BatchItemRequestModel;
use App\Models\AdminRoleModuleModel;
use App\Libraries\S3Library;
use App\Models\ItemMaterialModel;

class Order extends BaseController
{
	public function getAll()
	{

		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.order.order.main'),
			'menu' => 'order',
		];

		return view('order/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$orderModel = new OrderModel();

		$paramSearch = $this->request->getVar("param_search");

		$paramWhere = '';
		if($paramSearch == 1){
			$paramSearch = ' AND ( DATE_ADD(order_start_due_date, INTERVAL -7 DAY) <= DATE(NOW()) AND order_start_due_date >= DATE(NOw()) ) ';
		}

		$orderData = $orderModel->getAll([
			'where'=> $paramSearch.' AND order_status IN ("Proses", "New") AND order_active=1',
			'orderby'=> 'orders.order_start_due_date ASC'
		]);

		$data = [];

		if (@$orderData) {
        	foreach ($orderData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/order/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/order/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $btn_note = '&nbsp;<a href="'.base_url().'/order/note/'.$val->id.'" class="btn btn-warning btn-icon" target="_blank" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';

            $value['note'] = $val->note;
            $value['customer'] = $val->customer_name;
            $value['date'] = $val->date;
            $value['due_date'] = $val->due_date;
            $value['start_due_date'] = $val->start_due_date;
            $value['status'] = $val->status;
            $value['description'] = $val->description;
            $value['action'] = $btn_note.$btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

    public function noteOrder($date){
    	$orderModel = new OrderModel();
		$orderNota = $orderModel->setNota();

		if ($orderNota) {
			$number = $orderNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'OR/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
    }

	public function addOrder()
	{
		if (!$this->request->isAjax()) {

			$customerModel = new CustomerModel();

			$dataCustomer = $customerModel->getAll();

			$sizeModel = new SizeModel();
			$workflowModel = new WorkflowModel();
			$goldContentModel = new GoldContentModel();
			$categoryModel = new CategoryModel();
			$goldTypeModel = new GoldTypeModel();

			$data = [
				'data_customer' => $dataCustomer, 
				'title' => lang('Menu.order.order.main'),
				'menu' => 'order',
				'data_workflow' => $workflowModel->getAll(),
				'data_size' => $sizeModel->getAll(),
				'data_category' => $categoryModel->getAll(),
				'data_gold_content' => $goldContentModel->getAll(),
				'data_gold_type' => $goldTypeModel->getAll(),
			];

			return view('order/create', $data);
		}else{

			if (@$this->request->getGet('act') == 'addItemOrder') {
				return $this->addItemOrder();
			}

			$rules = [
	            'customer_id'             => ['label' => 'Pelanggan', 'rules' => 'required'],
	            'date'             => ['label' => 'Tanggal Order', 'rules' => 'required'],
	            'due_date'             => ['label' => 'Batas Tanggal Order', 'rules' => 'required'],
	            'start_due_date'        => ['label' => 'Batas Waktu Cor', 'rules' => 'required|max_length[255]'],
	            'request_total'          => ['label' => 'No Request Note', 'rules' => 'max_length[255]'],
	            'request_description'	=> ['label' => 'Keterangan Order', 'rules' => 'max_length[255]'],
	            'description'             => ['label' => 'Deskripsi', 'rules' => 'max_length[255]'],
	            'name' => ['label' => 'Barang', 'rules' => 'required'],
	        ];

	        if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$data = [
				'order_note'		    => $this->noteOrder($this->request->getPost('date')),
				'order_customer_id' 	=> $this->request->getPost('customer_id'),
				'order_date' 		    => $this->request->getPost('date'),
				'order_due_date' 		=> $this->request->getPost('due_date'),
				'order_start_due_date'	=> $this->request->getPost('start_due_date'),
				'order_request_total' => $this->request->getPost('request_total'),
				'order_request_description' => $this->request->getPost('request_description'),
				'order_description' 	=> $this->request->getPost('description'),
	            'order_insert_admin_id' => $this->session->get('id'),
	            'order_status'          => 'New',
	            'order_active'          => 1
	        ];

	        $id = $this->request->getPost('id');
	        $sku = $this->request->getPost('sku');
	        $name = $this->request->getPost('name');
	        $category_id = $this->request->getPost('category_id');
	        $gc_id = $this->request->getPost('gc_id');
	        $weight = $this->request->getPost('weight');
	        $size_id = $this->request->getPost('size_id');
	        $stone_type = $this->request->getPost('stone_type');
	        $quantity = $this->request->getPost('quantity');
	        $gt_id = $this->request->getPost('gt_id');
	        $print_no = $this->request->getPost('print_no');

	        $key = 0;
	        $dataPath = '';

	        $s3Library = new S3Library();
	        
	        $itemModel = new ItemModel();

			$data2 = [];
			
			$dataCheck  = [];

			if (isset($name)) {
				foreach ($name as $key => $value) {

					$rules['name.'.$key] = ['label' => 'Nama Barang', 'rules' => 'required'];
					$rules['category_id.'.$key] = ['label' => 'Kategori Barang', 'rules' => 'required|numeric'];
					$rules['gc_id.'.$key] = ['label' => 'Kadar Emas', 'rules' => 'required|numeric'];
					$rules['weight.'.$key] = ['label' => 'Berat Barang', 'rules' => 'required|numeric'];
					$rules['size_id.'.$key] = ['label' => 'Ukuran', 'rules' => 'trim'];
					// $rules['stone_type.'.$key] = ['label' => 'Tipe Batu', 'rules' => 'required'];
					$rules['quantity.'.$key] = ['label' => 'Jumlah', 'rules' => 'required|numeric'];
					$rules['gt_id.'.$key] = ['label' => 'Warna Emas', 'rules' => 'required|numeric'];
					$rules['print_no.'.$key] = ['label' => 'No Model', 'rules' => 'trim'];        

					$itemDataByID = $itemModel->getAll(['where' => ' AND item_id="'.@$id[$key].'" AND workflows.workflow_active=1 ']);

					if ($itemDataByID == null) {
						$msg = [
        	                'error' => 'Workflow SKU '.@$sku[$key].' tidak aktif, silahkan edit produk terlebih dahulu',
        	            ];
        	            return json_encode($msg);
					}

					if (@$id[$key] == '') {
					    
					    $itemDataBySKU = $itemModel->getBySKU($sku[$key]);
            		
            		
                		if (in_array($sku[$key], $dataCheck)) {
                				$msg = [
                	                'error' => lang('Validation.sku_already_exist'),
                	            ];
                	            return json_encode($msg);
                        }else{
                            $dataCheck[]= $sku[$key];
                        }
                		
                		
                        if ($itemDataBySKU) {
                				$msg = [
                	                'error' => lang('Validation.sku_already_exist'),
                	            ];
                	            return json_encode($msg);
                        }

						$fileData = @$this->request->getFileMultiple('image'.$key);

						if ($fileData[0]->getClientName() == '') {
							$msg = [
				                'error' =>  lang('Validation.image_required'),
				            ];
				            return json_encode($msg);
						}
							foreach ($fileData as $key2 => $value2) {
									$rules['image'.$key.'.'.$key2] = ['label' => 'Image', 'rules' => 'uploaded[image'.$key.'.'.$key2.']|mime_in[image'.$key.'.'.$key2.',image/jpg,image/jpeg,image/gif,image/png]|max_size[image'.$key.'.'.$key2.',4096]'];
							}

						if(count($fileData)> 3){
							$msg = [
				                'error' =>  lang('Validation.max_count_file'),
				            ];
				            return json_encode($msg);
						}
					}

				}
			}
			if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$ItemMaterialModel = new ItemMaterialModel();

			$dataStone = [];

			foreach ($name as $key => $value) {
				if (@$id[$key]) {
					$param['where'] = ' AND im_item_id="'.$id[$key].'" AND im_active=1 ';
					$itemMaterialData = $ItemMaterialModel->getAll($param); 


					foreach ($itemMaterialData as $key1 => $value1) {
						
						if (@$value1->source == 'Gudang Batu') {	

							if (!array_key_exists("material_bulk[".$value1->name."]", $dataStone)) {
									$dataStone["material_bulk[".$value1->name."]"] = 0;
							}
								
							$dataStone["material_bulk[".$value1->name."]"] += intval($value1->quantity)*$quantity[$key];
						}
					}
					if (!empty($dataStone)) {
						$dataStone["user_id"] = 1;
						$dataStone["description"] = 'Nota Order : '.$data['order_note'];
					}
				}
			}

			if (!empty($dataStone)) {
				$api['headers'] = [
					'Accept'        => 'application/json',
					'Authorization: Bearer '.GUDANG_BATU_TOKEN
				];

				$resultMaterialOut = json_decode(sendRequest(GUDANG_BATU_URL_VALIDATE, 'POST', $api['headers'], $dataStone ));
				
				if ($resultMaterialOut->status != 200) {
					return result_message_json($resultMaterialOut, 'add', 'gudang_batu');
				}
			}


			$orderModel = new OrderModel();
			$orderItemModel = new OrderItemModel();

			$orderDataByNota = $orderModel->getByNota($data['order_note']);
	        if (count($orderDataByNota) > 0) {
				$msg = [
	                'error' => lang('Validation.nota_already_exist'),
	            ];
	            return json_encode($msg);
	        }

			$orderID = $orderModel->addOrder($data);

			$dataOi = [];
			foreach ($name as $key => $value) {

				$fileNameInsert = [];

				if (@$id[$key] == '') {

					if ($this->request->getFileMultiple('image'.$key) != null) {

						$fileData = $this->request->getFileMultiple('image'.$key);

						foreach ($fileData as $key2 => $value2) {

							$fileName = preg_replace("/[^a-zA-Z0-9]+/", "", $fileData[$key2]->getClientName());
				            $extension = $fileData[$key2]->guessExtension();
				            $type = $fileData[$key2]->getMimeType();
				            $path = getenv('AWS_REPOSITORY_FOLDER').'/'.$this->session->get('id').'/'.time().'_'.$fileName;

				            $fileNameInsert[] = $path;

				            $s3Library->putObject($path,$fileData[$key2]);
						}
					}

					$dataItem =[
						'item_sku' => $sku[$key],
						'item_name' => $name[$key],
						'item_category_id' => $category_id[$key],
						'item_gc_id' => $gc_id[$key],
						'item_gt_id' => $gt_id[$key],
						'item_print_no' => $print_no[$key],
						'item_weight' => $weight[$key],
						'item_size_id' => $size_id[$key],
						'item_stone_type' => $stone_type[$key],
			            'item_insert_admin_id' => $this->session->get('id'),
			            'item_image_1' => json_encode($fileNameInsert),
			            'item_active'  => 1
					];

					$itemID = $itemModel->addItem($dataItem);
				}else{
					$itemID = $id[$key];
				}

				$dataOi[] = [
					'oi_order_id' => $orderID,
					'oi_item_id' => $itemID,
					'oi_date'=> $this->request->getPost('date'),
					'oi_due_date' => $this->request->getPost('due_date'),
					'oi_item_name' => $name[$key],
					'oi_gc_id' => $gc_id[$key],
					'oi_size_id' => $size_id[$key],
					'oi_weight' => $weight[$key],
					'oi_quantity' => $quantity[$key],
					'oi_image_1' => json_encode($fileNameInsert),
					'oi_status' => 'New',
		            'oi_insert_admin_id' => $this->session->get('id'),
		            'oi_active'          => 1
				];


				if (@$id[$key] == '') {
					$param['where'] = ' AND im_item_id="'.$id[$key].'" AND im_active=1 ';
					$itemMaterialData = $ItemMaterialModel->getAll($param); 

					foreach ($itemMaterialData as $key => $value) {

						if ($value->required_to_check == 1) {
							$dataStone[] = [	
								"material_bulk[".$value->name."]" => $value->quantity
							];
						}
					}
				}
			}

			$orderItemData = $orderItemModel->addOrderItemBulk($dataOi);

			$adminRoleModuleModel = new AdminRoleModuleModel();

			$query = [
				'section' => 'Order',
				'groupby' => 'admin_phone'
			];
	        $adminRoleModuleData = $adminRoleModuleModel->getAllWithModule($query);

	   //      foreach ($adminRoleModuleData as $key => $value) {
				// $result = sendWhatsApp($value->admin_phone, 'New Order | NOTA : '.$data['order_note']);
	   //      }

	        // sendWhatsApp('088232340119', 'New Order | NOTA : '.$data['order_note']);
	        // sendWhatsApp('08122680110', 'New Order | NOTA : '.$data['order_note']);
	        // sendWhatsApp('081390548982', 'New Order | NOTA : '.$data['order_note']);


			if ($orderID) {
				$data = [
					'success'=> lang('Validation.success_add'),
					'redirect'=> 'order'
				];

				return json_encode($data);
			}

		}

	}

	public function updateOrder($id)
	{

		if (!$this->request->isAjax()) {


			$orderModel = new OrderModel();

			$dataOrder = $orderModel->getByID($id);

			$orderItemModel = new OrderItemModel();

			$dataOrderItem = $orderItemModel->getByOrderID($id);


			$batchItemRequestModel = new BatchItemRequestModel();
			$batchItemRequestData = $batchItemRequestModel->getAll(['where'=> ' AND  order_items.oi_order_id="'.$id.'" AND (bir_bit_id IS NULL OR bir_bit_id="") AND bir_active=1 ']);

			$batchItemRequestArray = [];

			if(count($batchItemRequestData) > 0){
				foreach ($batchItemRequestData as $key => $value) {
					$batchItemRequestArray[$value->oi_item_id][] = $value->item_sku;
				}
			}else{
				$itemMaterialModel = new ItemMaterialModel();

				$itemArray = [];
				foreach ($dataOrderItem as $key1 => $value1) {
				 	$itemArray[] = $value1->item_id;
				}

				if (!empty($itemArray)) {
					$itemMaterialData = $itemMaterialModel->getAll(['where' => ' AND im_item_id IN ('.implode(',', $itemArray).') AND im_active=1 ']);
					foreach ($itemMaterialData as $value_) {
						$batchItemRequestArray[$value_->item_id][] = $value_->name;
					}
				}
			}

			$data = [
				'data' => (isset($dataOrder))? $dataOrder : [],
				'data_order_item' => $dataOrderItem,
				'title' => lang('Menu.order.order.main'),
				'batch_item_request' => $batchItemRequestArray,
				'menu' => 'order',
			];

			return view('order/edit',  $data);

		}else{

			$rules = [
	            'date'             		=> ['label' => 'Order date', 'rules' => 'required'],
	            'due_date'             	=> ['label' => 'Order due date', 'rules' => 'required'],
	            'description'           => ['label' => 'Order description', 'rules' => 'max_length[255]'],
	            'start_due_date'        => ['label' => 'Batas Waktu Cor', 'rules' => 'required|max_length[255]'],
	            'request_total'          => ['label' => 'No Request Note', 'rules' => 'max_length[255]'],
	            'request_description'	=> ['label' => 'Keterangan Order', 'rules' => 'max_length[255]'],
	        ];

	        if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$orderModel = new OrderModel();

			$data = [
				'order_date' 		    => $this->request->getPost('date'),
				'order_due_date' 		=> $this->request->getPost('due_date'),
				'order_start_due_date'	=> $this->request->getPost('start_due_date'),
				'order_request_total' => $this->request->getPost('request_total'),
				'order_request_description' => $this->request->getPost('request_description'),
				'order_description' 	=> $this->request->getPost('description'),
	            'order_update_admin_id' => $this->session->get('id'),
	            'order_active'          => 1
	        ];

	        if ($this->request->getPost('status') == 'Completed') {
	        	$data['order_status'] = 'Completed';
	        }

			$orderData = $orderModel->updateOrder($data, $id); 

			if ($orderData) {
				$data = [
					'success'=> lang('Validation.success_edit'),
					'redirect' => 'order'
				];

				return json_encode($data);
			}

		}
	}

	public function deleteOrder($id)
	{

		$batchItemModel = new BatchItemModel();
		if(count($batchItemModel->getAll(["where" => ' AND order_items.oi_order_id="'.$id.'" AND bi_active=1 AND batch_status != "Cancelled" '])) > 0 ){
			$data = [
				'error'=> lang('Validation.order_item_already_use'),
			];
			return json_encode($data);
		}

		$orderModel = new OrderModel();
		$orderItemModel = new OrderItemModel();

		$orderData = $orderModel->deleteOrder($id, $this->session->get('id')); 

		$orderItemData = $orderItemModel->deleteOrderItemByOrderID($id, $this->session->get('id')); 

		if ($orderData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

	public function addItemOrder(){


		$id = $this->request->getPost('item_code');
		$noindex = $this->request->getPost('noindex');

		$itemModel = new ItemModel();

		$dataItem = $itemModel->getBySKU($id);

		if ($dataItem == null) {
			$data = [
				'error'=> lang('Validation.data_not_found'), 
			];

			return json_encode($data);
		}

		$dataItem = $dataItem[0];

		$itemMaterialModel = new ItemMaterialModel();
		$itemMaterialData = $itemMaterialModel->getAll(['where' => 'AND im_item_id="'.$dataItem->id.'" AND im_active=1']);
		$itemMaterialArray = [];
		foreach ($itemMaterialData as $value_) {
			$itemMaterialArray[] = $value_->name;
		}


		$stoneType = implode(", ", $itemMaterialArray);

		$html = '<div class="card card-xxl-stretch mb-5 item-card">
				<div class="card-body">
					<div class="row">
					<div class="col-md-11">
					</div>
					<div class="col-md-1">
						<button type="button" style="margin-left:20px" data-item_code="'.$id.'" class="btn btn-danger btn-icon btn-sm remove-item"><i class="fa fa-trash"></i></button>
					</div>
						<div class="form-group my-3 col-md-4">
							<label>SKU</label>
							<input readonly="" type="text" name="sku['.$noindex.']"  value="'.$dataItem->sku.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<input type="hidden" name="id['.$noindex.']" value="'.$dataItem->id.'">
							<label>Nama</label>
							<input readonly="" type="text" name="name['.$noindex.']"  value="'.$dataItem->name.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Kategori</label>
							<input readonly="" type="hidden" name="category_id['.$noindex.']" value="'.$dataItem->category_id.'" class="form-control form-control-solid mt-1" >
							<input readonly="" type="text"  value="'.$dataItem->category_name.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Kadar Emas</label>
							<input readonly="" name="gc_id['.$noindex.']" type="hidden" value="'.$dataItem->gc_id.'" class="form-control form-control-solid mt-1" placeholder="">
							<input readonly="" type="text"  value="'.$dataItem->gc_name.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Warna Emas</label>
							<input readonly="" name="gt_id['.$noindex.']" type="hidden" value="'.$dataItem->gt_id.'" class="form-control form-control-solid mt-1" placeholder="">
							<input readonly="" type="text"  value="'.$dataItem->gt_name.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Berat</label>
							<input readonly="" type="text" name="weight['.$noindex.']"  value="'.$dataItem->weight.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Ukuran</label>
							<input readonly="" name="size_id['.$noindex.']" type="hidden" value="'.$dataItem->size_id.'" class="form-control form-control-solid mt-1" placeholder="">
							<input readonly=""  value="'.$dataItem->size_name.'" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Jenis Batu</label>
							<input readonly="" type="text" value="'.$stoneType.'" name="stone_type['.$noindex.']" class="form-control form-control-solid mt-1" placeholder="">
						</div>
						<div class="form-group my-3 col-md-4">
							<label>Quantity</label>
							<input type="number" name="quantity['.$noindex.']" class="form-control mt-1" placeholder="">
							<input type="file" name="image'.$noindex.'[]" multiple="true" hidden class="form-control mt-1 image" accept="image/*" placeholder="">
						</div>
				</div>
			</div>';

		$data = [
			'success'=> $html
		];

		return json_encode($data);
	}


	public function note($id)
	{
		$orderModel = new OrderModel();
		$orderData = $orderModel->getByID($id);
		$orderItemModel = new OrderItemModel();
		$orderItemData = $orderItemModel->getByOrderID($id);

		$batchItemRequestModel = new BatchItemRequestModel();
		$batchItemRequestData = $batchItemRequestModel->getAll(['where'=> ' AND  order_items.oi_order_id="'.$id.'" AND (bir_bit_id IS NULL OR bir_bit_id="") AND bir_active=1 ']);

		// dd($batchItemRequestData);

		$batchItemRequestArray = [];

		if(count($batchItemRequestData) > 0){
			foreach ($batchItemRequestData as $key => $value) {
				$batchItemRequestArray[$value->oi_item_id][] = $value->item_sku;
			}
		}else{
			$itemMaterialModel = new ItemMaterialModel();

			$itemArray = [];
			foreach ($orderItemData as $key1 => $value1) {
			 	$itemArray[] = $value1->item_id;
			}
			
			if (!empty($itemArray)) {
				$itemMaterialData = $itemMaterialModel->getAll(['where' => ' AND im_item_id IN ('.implode(',', $itemArray).') AND im_active=1 ']);
				foreach ($itemMaterialData as $value_) {
					$batchItemRequestArray[$value_->item_id][] = $value_->name;
				}
			}
		}

		// dd($batchItemRequestArray);

		if (!$this->request->isAjax()) {

			$data = [
				'data' => (isset($orderData))? $orderData : [],
				'data_order_item' => $orderItemData,
				'title' => lang('Menu.order.order.main'),
				'batch_item_request' => $batchItemRequestArray,
				'menu' => 'order',
			];

			return view('order/note',  $data);
		}
	}
}