<?php
namespace App\Controllers;

use App\Models\ItemModel;
use App\Models\OutgoingNoteModel;
use App\Models\OutgoingNoteDetailModel;
use App\Models\BatchModel;
use App\Models\BatchItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Models\AdminRoleModuleModel;
use App\Models\TreeModel;
use App\Models\PrintHistoryModel;
use App\Models\GoldContentModel;
use App\Models\ItemMaterialModel;
use App\Models\BatchItemRequestModel;
use App\Models\BatchItemTaskModel;
use App\Models\TaskModel;
use App\Models\BatchItemOutputModel;

class OutgoingBatch extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){

			if (@$this->request->getGet('act') == 'batchItemData') {
				return $this->batchItemData();
			}

			return $this->data();
		}

		
		$data = [
			'title' => lang('Menu.order.outgoing-batch.main'),
			'menu' => 'outgoing-batch',
		];

		return view('outgoing-batch/index', $data);
	}

	public function batchItemData(){

		$rules = [
            'from_date'        => ['label' => 'Dari Tanggal', 'rules' => 'required|max_length[255]'],
            'to_date'          => ['label' => 'Sampai Tanggal', 'rules' => 'required|max_length[255]'],
            'note'          => ['label' => 'No Order/Batch', 'rules' => 'max_length[25]'],
            // 'task_id'           => ['label' => 'Divisi', 'rules' => 'required|max_length[255]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
		$toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
		$note = $this->request->getVar('note', FILTER_SANITIZE_STRING);
		$whereNote = '';
		if (!empty($note)){
			$whereNote = ' AND (batch_code LIKE "%'.$note.'%" OR order_note LIKE "%'.$note.'%" ) ';
		}
		// $taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);

		$batchItemOutputModel = new BatchItemOutputModel();
		$batchItemData = $batchItemOutputModel->getAll(['where' => ' AND DATE(bio_insert_at) >= "'.$fromDate.'" AND DATE(bio_insert_at) <=  "'.$toDate.'" '.$whereNote.' AND bio_active=1 AND bio_on_id IS NULL AND bio_is_melt != 1 ']);

		// $batchItemData = $batchItemOutputModel->getAll(['where' => ' AND DATE(bi_update_at) >= "'.$fromDate.'" AND DATE(bi_update_at) <=  "'.$toDate.'" '.$whereNote.' AND bi_completed_quantity IS NOT NULL AND bi_completed_weight IS NOT NULL AND bio_active=1 AND bio_on_id IS NULL ']);

		if (empty($batchItemData)) {
			$msg = [
                'error' => lang('Validation.batchProcessEmpty'),
            ];
            return json_encode($msg);
		}

		$html = '';

		foreach ($batchItemData as $key => $value) {
			$html .= '
			<tr>
				<td>SKU : '.$value->item_sku.'<br>
				Barang : 
				<input type="hidden" name="names['.$value->id.']" value="'.$value->item_name.'">
				<input type="hidden" name="batch_item['.$value->id.']" value="'.$value->bi_id.'">
				'.$value->item_name.'<br>
				Pelanggan : '.$value->customer_name.'<br>
				Alur Kerja : '.$value->workflow_name.'</td>
				<td>
					Nota Order : '.$value->order_note.'<br>
					Kode Batch : '.$value->batch_code.'
 				</td>
				<td>'.$value->group_note.'</td>
				<td>'.(($value->is_melt == 1)? "Lebur" : "").'</td>
				<td>
					<input type="number" step="any" readonly value="'.$value->item_quantity.'" name="batch_item_quantity['.$value->id.']" class="form-control bg-light" style="width:100px">
				</td>
				<td>
					<input type="number" step="any" readonly value="'.$value->item_weight.'" name="batch_item_weight['.$value->id.']" class="form-control bg-light" style="width:100px">
				</td>
				<td>
					<input type="checkbox" name="batch_item_output[]" value="'.$value->id.'">
				</td>
			</tr>
			';
		}

		// Tanggal Mulai : '.$value->bit_start_at.'<br>
		// Tanggal Selesai : '.$value->bit_end_at.'<br>

		return json_encode([
            'success' => $html,
        ]);
	}


	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}

		$paramType = $this->request->getGet('paramType');
		if ($paramType == 'treeByID') {
			return $this->treeByID();
		}else if($paramType == 'gcByID'){
			return $this->gcByID();
		}
		
		$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteData = $outgoingNoteModel->getAll([
			'where' => ' AND on_types="Barang Baik" AND on_active=1 ',
			'orderby'=> 'on_insert_at DESC'
		]);

		$data = [];

		if (@$outgoingNoteData) {
        	foreach ($outgoingNoteData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/outgoing-batch/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_print = '&nbsp;<a href="'.base_url().'/outgoing-batch/print/'.$val->id.'" class="btn btn-warning btn-icon" target="_blank" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';

	            $value['notes'] = $val->notes.'<br/>'.$btn_print;
	            $value['remark'] = $val->remark;
	            $value['date'] = $val->date;
	            $value['start_date'] = $val->start_date;
	            $value['end_date'] = $val->end_date;
	            $value['task_name'] = $val->task_name;
	            $value['remark'] = $val->remark;
	            $value['total_quantity'] = $val->total_quantity;
	            $value['total_weight'] = $val->total_weight;
	            $value['status'] = $val->status;
	            $value['action'] = $btn_print;

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addOutgoingBatch()
	{
		if (!$this->request->isAjax()) {

			$taskModel = new TaskModel();
			
			$data = [
				'title' => lang('Menu.order.outgoing-batch.main'),
				'menu' => 'outgoing-batch',
				'data_task' => $taskModel->getAll()
			];

			return view('outgoing-batch/create', $data);
		}else{

			// on_id as "id", on_notes as "notes", on_date as "date", on_remark as "remark", on_total_quantity as "total_quantity", on_total_weight as "total_weight", on_status as "status", on_insert_at as "insert_at"

			$fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
			$toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
			$taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);
			$batchItemOutput = $this->request->getVar('batch_item_output');

			$rules = [
	            'from_date'    => ['label' => 'Dari Tanggal', 'rules' => 'required|max_length[255]'],
	            'to_date'      => ['label' => 'Sampai Tanggal', 'rules' => 'required|max_length[255]'],
	            'remark'       => ['label' => 'Deskripsi', 'rules' => 'max_length[255]'],
	            // 'batch_item_id.0' => ['label' => 'Batch Item', 'rules' => 'required'],
	        ];


			$outgoingNoteDetailModel = new OutgoingNoteDetailModel();
			$outgoingNoteModel = new OutgoingNoteModel();

	        $names = $this->request->getVar('names');
			$weights = $this->request->getVar('weights');
			$quantities = $this->request->getVar('quantities');

			if (empty($batchItemOutput)) {
				$data = [
					'error'=> lang('Validation.weightRequired'),
				];
				return json_encode($data);
			}

			if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			$bitWeight = $this->request->getVar('batch_item_weight');
			$bitQuantity = $this->request->getVar('batch_item_quantity');

			$batchItem = $this->request->getVar('batch_item', FILTER_SANITIZE_STRING);


			$bitWeightCheck = [];
			if (isset($bitWeight)) {
				foreach ($bitWeight as $key => $value) {
					if (!empty($value)) {
						$bitWeightCheck[] = $value;
					}
				}
			}

			if (empty($bitWeightCheck)) {
				$data = [
					'error'=> lang('Validation.weightRequired'),
				];
				return json_encode($data);
			}

			$bitWeightQty = 0;
			$bitWeightWeight = 0;

			$batchItemOutputModel = new BatchItemOutputModel();
			foreach($batchItemOutput as $key  => $value){
				$batchItemOutputData = $batchItemOutputModel->getAll(['where' => 'AND bio_id ="'.$value.'" ']);
				if ($value != '') {
					$bitWeightQty += @$batchItemOutputData[0]->item_quantity;
					$bitWeightWeight += @$batchItemOutputData[0]->item_weight;
				}
			}


			$data = [
				'on_notes'		    	=> $this->noteOutgoingNote(date('Y-m-d')),
				'on_date' 				=> date('Y-m-d'),
				'on_types'				=> 'Barang Baik',
				'on_start_date' 		=> $fromDate, 
				'on_end_date' 			=> $toDate, 
				'on_task_id' 			=> NULL, 
				'on_remark' 			=> $this->request->getPost('remark'), 
				'on_total_quantity' 	=> $bitWeightQty,
				'on_total_weight' 		=> $bitWeightWeight,
				'on_status' 			=> '',
				'on_insert_At' 			=> getCurrentTime(),
				'on_insert_admin_id' 	=> $this->session->get('id'),
	            'on_active'          	=> 1
	        ];
			
			$onID = $outgoingNoteModel->addOutgoingNote($data);

			$ongoingDetail = [];

			foreach($batchItemOutput as $key  => $value){
				$batchItemOutputData = $batchItemOutputModel->getAll(['where' => 'AND bio_id ="'.$value.'" ']);
				if ($value != '') {
					$ongoingDetail[] = [
						'ond_on_id' => $onID,
						'ond_bi_id' => @$batchItemOutputData[0]->bi_id,
						'ond_bit_id' => @$batchItemOutputData[0]->bit_id,
						'ond_name' => @$batchItemOutputData[0]->item_name,
						'ond_quantity' =>@$batchItemOutputData[0]->item_quantity,
						'ond_weight' => @$batchItemOutputData[0]->item_weight,
						'ond_is_melt' => @$batchItemOutputData[0]->is_melt,
						'ond_insert_at' => getCurrentTime(),
						'ond_insert_admin_id' => $this->session->get('id'),
					];

					$batchItemOutputModel->updateBatchItemOutput([
						'bio_on_id' => $onID,
						'bio_update_at' => getCurrentTime(),
						'bio_update_admin_id' => $this->session->get('id'),
					], $value);
				}
			}

			$ondID = $outgoingNoteDetailModel->addOutgoingNoteDetailBulk($ongoingDetail);

			if ($onID && $ondID) {
				$data = [
					'success'=> lang('Validation.success_add'),
					'redirect'=> 'outgoing-batch'
				];

				return json_encode($data);
			}

		}

	}

	public function print($id){

		$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteDetailModel = new OutgoingNoteDetailModel();

		$outgoingNoteData = $outgoingNoteModel->getAll([
			'where' => ' AND on_id="'.$id.'" ',
			'orderby'=> 'on_id DESC'
		]);


		$outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([
			'where' => ' AND ond_on_id="'.$id.'" ',
			'orderby'=> ' ond_id ASC'
		]);

		$data = [
			'title' => lang('Menu.order.outgoing-batch.main'),
			'menu' => 'outgoing-batch',
			'data' => $outgoingNoteData[0],
			'data_item' => $outgoingNoteDetailData
		];

		return view('outgoing-batch/print', $data);

	}


	public function deleteOutgoingBatch($id)
	{

		// $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
		// $outgoingNoteModel = new OutgoingNoteModel();

		// $outgoingNoteData = $outgoingNoteModel->deleteOutgoingNote($id, $this->session->get('id'));

		// if ($outgoingNoteData) {
		// 	$data = [
		// 		'success'=> lang('Validation.success_delete'),
		// 	];

		// 	return json_encode($data);
		// }else{
		// 	$data = [
		// 		'error'=> lang('Validation.error_delete'),
		// 	];

		// 	return json_encode($data);
		// }
	}


	public function noteOutgoingNote($date){
    	$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteNota = $outgoingNoteModel->setNota();

		if ($outgoingNoteNota) {
			$number = $outgoingNoteNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'STB/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
    }
	
}