<?php
namespace App\Controllers;

use App\Models\ItemModel;
use App\Models\OutgoingNoteModel;
use App\Models\OutgoingNoteDetailModel;
use App\Models\BatchModel;
use App\Models\BatchItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Models\AdminRoleModuleModel;
use App\Models\TreeModel;
use App\Models\PrintHistoryModel;
use App\Models\GoldContentModel;
use App\Models\ItemMaterialModel;
use App\Models\BatchItemRequestModel;
use App\Models\BatchItemTaskModel;
use App\Models\TaskModel;

class OutgoingNote extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){

			if (@$this->request->getGet('act') == 'batchItemData') {
				return $this->batchItemData();
			}

			return $this->data();
		}

		
		$data = [
			'title' => lang('Menu.order.outgoing-note.main'),
			'menu' => 'outgoing-note',
		];

		return view('outgoing-note/index', $data);
	}

	public function batchItemData(){

		$rules = [
            'from_date'        => ['label' => 'Dari Tanggal', 'rules' => 'required|max_length[255]'],
            'to_date'          => ['label' => 'Sampai Tanggal', 'rules' => 'required|max_length[255]'],
            'task_id'           => ['label' => 'Divisi', 'rules' => 'required|max_length[255]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
		$toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
		$taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemData = $batchItemTaskModel->getAll(['where' => ' AND bit_status="Completed" AND bit_task_id="'.$taskID.'" AND ( DATE(bit_end_at) >="'.$fromDate.'" AND DATE(bit_end_at) <="'.$toDate.'") AND bit_repaired_bit_id IS NULL AND bi_active=1 ']);
		

		if (empty($batchItemData)) {
			$msg = [
                'error' => lang('Validation.batchProcessEmpty'),
            ];
            return json_encode($msg);
		}

		$html = '';

		foreach ($batchItemData as $key => $value) {
			$html .= '
			<tr>
				<td>SKU : '.$value->item_sku.'<br>
				Barang : 
				<input type="hidden" name="names['.$value->id.']" value="'.$value->item_name.'">
				<input type="hidden" name="batch_item['.$value->id.']" value="'.$value->bi_id.'">
				'.$value->item_name.'<br>
				Pelanggan : '.$value->customer_name.'<br>
				Alur Kerja : '.$value->workflow_name.'</td>
				<td>
					Nota Order : '.$value->order_note.'<br>
					Kode Batch : '.$value->batch_code.'
 				</td>
				<td>'.$value->group_note.'</td>
				<td>
					<input type="number" step="any" name="batch_item_weight['.$value->id.']" class="form-control" style="width:200px">
				</td>
			</tr>
			';
		}

		// Tanggal Mulai : '.$value->bit_start_at.'<br>
		// Tanggal Selesai : '.$value->bit_end_at.'<br>

		return json_encode([
            'success' => $html,
        ]);
	}


	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}

		$paramType = $this->request->getGet('paramType');
		if ($paramType == 'treeByID') {
			return $this->treeByID();
		}else if($paramType == 'gcByID'){
			return $this->gcByID();
		}
		
		$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteData = $outgoingNoteModel->getAll([
			'where' => ' AND on_types="Bahan Sisa" AND on_active=1 ',
			'orderby'=> 'on_insert_at DESC'
		]);

		$data = [];

		if (@$outgoingNoteData) {
        	foreach ($outgoingNoteData as $index => $val ) {
        		
	            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/outgoing-note/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

	            $btn_print = '&nbsp;<a href="'.base_url().'/outgoing-note/print/'.$val->id.'" class="btn btn-warning btn-icon" target="_blank" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';

	            $value['notes'] = $val->notes.'<br/>'.$btn_print;
	            $value['remark'] = $val->remark;
	            $value['date'] = $val->date;
	            $value['start_date'] = $val->start_date;
	            $value['end_date'] = $val->end_date;
	            $value['task_name'] = $val->task_name;
	            $value['remark'] = $val->remark;
	            $value['total_quantity'] = $val->total_quantity;
	            $value['total_weight'] = $val->total_weight;
	            $value['status'] = $val->status;
	            $value['action'] = $btn_print.$btn_delete;

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addOutgoingNote()
	{
		if (!$this->request->isAjax()) {

			$taskModel = new TaskModel();
			
			$data = [
				'title' => lang('Menu.order.outgoing-note.main'),
				'menu' => 'outgoing-note',
				'data_task' => $taskModel->getAll()
			];

			return view('outgoing-note/create', $data);
		}else{

			$paramType = $this->request->getVar('param', FILTER_SANITIZE_STRING);

			if ($paramType == 'addOutgoingNoteProcess') {
				return $this->addOutgoingNoteProcess();
			}

			// on_id as "id", on_notes as "notes", on_date as "date", on_remark as "remark", on_total_quantity as "total_quantity", on_total_weight as "total_weight", on_status as "status", on_insert_at as "insert_at"

			$fromDate = $this->request->getVar('from_date', FILTER_SANITIZE_STRING);
			$toDate = $this->request->getVar('to_date', FILTER_SANITIZE_STRING);
			$taskID = $this->request->getVar('task_id', FILTER_SANITIZE_STRING);

			$rules = [
	            'from_date'    => ['label' => 'Dari Tanggal', 'rules' => 'required|max_length[255]'],
	            'to_date'      => ['label' => 'Sampai Tanggal', 'rules' => 'required|max_length[255]'],
	            'task_id'      => ['label' => 'Divisi', 'rules' => 'required|max_length[255]'],
	            'remark'       => ['label' => 'Deskripsi', 'rules' => 'max_length[255]'],
	            // 'batch_item_id.0' => ['label' => 'Batch Item', 'rules' => 'required'],
	        ];


			$outgoingNoteDetailModel = new OutgoingNoteDetailModel();
			$outgoingNoteModel = new OutgoingNoteModel();

	        $names = $this->request->getVar('names', FILTER_SANITIZE_STRING);
			$weights = $this->request->getVar('weights', FILTER_SANITIZE_STRING);
			$quantities = $this->request->getVar('quantities', FILTER_SANITIZE_STRING);

			if ($taskID == '-') {


				if (empty($names)) {
					$data = [
						'error'=> lang('Validation.weightRequired'),
					];
					return json_encode($data);
				}

				foreach ($names as $key => $value) {
					$rules['names.'.$key] = ['label' => 'Nama Ke '.($key+1), 'rules' => 'required|max_length[255]'];
					$rules['weights.'.$key] = ['label' => 'Berat Ke '.($key+1), 'rules' => 'required|max_length[255]'];
					$rules['quantities.'.$key] = ['label' => 'Jumlah Ke '.($key+1), 'rules' => 'required|max_length[255]'];
				}

			}

			if (!$this->validate($rules)) {
				$msg = [
	                'error' => $this->validation->listErrors(),
	            ];
	            return json_encode($msg);
			}

			if ($taskID == '-') {


				$data = [
					'on_notes'		    	=> $this->noteOutgoingNote(date('Y-m-d')),
					'on_date' 				=> date('Y-m-d'),
					'on_types'				=> 'Bahan Sisa',
					'on_start_date' 		=> $fromDate, 
					'on_end_date' 			=> $toDate, 
					'on_task_id' 			=> $taskID, 
					'on_remark' 			=> $this->request->getPost('remark'), 
					'on_total_quantity' 	=> array_sum($quantities),
					'on_total_weight' 		=> array_sum($weights),
					'on_status' 			=> '',
					'on_insert_At' 			=> getCurrentTime(),
					'on_insert_admin_id' 	=> $this->session->get('id'),
		            'on_active'          	=> 1
		        ];
				
				$onID = $outgoingNoteModel->addOutgoingNote($data);

				$ongoingDetail = [];
				foreach($names as $key  => $value){
					if ($value != '') {
						$ongoingDetail[] = [
							'ond_on_id' => $onID,
							'ond_bi_id' => NULL,
							'ond_name' => @$names[$key],
							'ond_quantity' => @$quantities[$key],
							'ond_weight' => @$weights[$key],
							'ond_insert_at' => getCurrentTime(),
							'ond_insert_admin_id' => $this->session->get('id'),
						];
					}
				}

				$ondID = $outgoingNoteDetailModel->addOutgoingNoteDetailBulk($ongoingDetail);

				if ($onID && $ondID) {
					$data = [
						'success'=> lang('Validation.success_add'),
						'redirect'=> 'outgoing-note'
					];

					return json_encode($data);
				}


				
			}else{

				$bitWeight = $this->request->getVar('batch_item_weight');
				$batchItem = $this->request->getVar('batch_item', FILTER_SANITIZE_STRING);


				$bitWeightCheck = [];
				if (isset($bitWeight)) {
					
					foreach ($bitWeight as $key => $value) {
						if (!empty($value)) {
							$bitWeightCheck[] = $value;
						}
					}

				}

				if (empty($bitWeightCheck)) {
					$data = [
						'error'=> lang('Validation.weightRequired'),
					];
					return json_encode($data);
				}


				$data = [
					'on_notes'		    	=> $this->noteOutgoingNote(date('Y-m-d')),
					'on_date' 				=> date('Y-m-d'),
					'on_types'				=> 'Bahan Sisa',
					'on_start_date' 		=> $fromDate, 
					'on_end_date' 			=> $toDate, 
					'on_task_id' 			=> $taskID, 
					'on_remark' 			=> $this->request->getPost('remark'), 
					'on_total_quantity' 	=> count($bitWeightCheck),
					'on_total_weight' 		=> array_sum($bitWeightCheck),
					'on_status' 			=> '',
					'on_insert_At' 			=> getCurrentTime(),
					'on_insert_admin_id' 	=> $this->session->get('id'),
		            'on_active'          	=> 1
		        ];
				
				$onID = $outgoingNoteModel->addOutgoingNote($data);

				$ongoingDetail = [];
				foreach($bitWeight as $key  => $value){
					if ($value != '') {
						$ongoingDetail[] = [
							'ond_on_id' => $onID,
							'ond_bi_id' => @$batchItem[$key],
							'ond_bit_id' => $key,
							'ond_name' => '',
							'ond_quantity' => 1,
							'ond_weight' => $value,
							'ond_insert_at' => getCurrentTime(),
							'ond_insert_admin_id' => $this->session->get('id'),
						];
					}
				}

				$ondID = $outgoingNoteDetailModel->addOutgoingNoteDetailBulk($ongoingDetail);

				if ($onID && $ondID) {
					$data = [
						'success'=> lang('Validation.success_add'),
						'redirect'=> 'outgoing-note'
					];

					return json_encode($data);
				}
			}

		}

	}

	public function addOutgoingNoteProcess(){

		$rules = [
			'quantity.0'  => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'weight.0'    => ['label' => 'Berat', 'rules' => 'required|max_length[11]'],
            'name.0'      => ['label' => 'Nama', 'rules' => 'required|max_length[11]'],
            'bit_id'      => ['label' => 'Batch item task ID', 'rules' => 'required|max_length[255]'],
            'remark'      => ['label' => 'Deskripsi', 'rules' => 'max_length[255]'],
            'datetime'    => ['label' => 'Waktu Akhir Pengerjaan', 'rules' => 'required|max_length[255]'],
        ];


		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$bitQuantity = $this->request->getVar('quantity', FILTER_SANITIZE_STRING);
		$bitWeight = $this->request->getVar('weight', FILTER_SANITIZE_STRING);
		$bitName = $this->request->getVar('name', FILTER_SANITIZE_STRING);
		$bitID = $this->request->getVar('bit_id', FILTER_SANITIZE_STRING);
		$datetime = $this->request->getVar('datetime', FILTER_SANITIZE_STRING);

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$bitID.'" AND bit_id AND bit_repaired_bit_id IS NULL AND bi_active=1 ']);

		$data = [
			'on_notes'		    	=> $this->noteOutgoingNoteProcess(date('Y-m-d')),
			'on_date' 				=> date('Y-m-d'),
			'on_types'				=> 'Bahan Sisa',
			'on_start_date' 		=> $datetime, 
			'on_end_date' 			=> $datetime,
			'on_task_id' 			=> $batchItemTaskData[0]->task_id, 
			'on_remark' 			=> $this->request->getPost('remark'), 
			'on_total_quantity' 	=> array_sum($bitQuantity),
			'on_total_weight' 		=> array_sum($bitWeight),
			'on_status' 			=> '',
			'on_insert_At' 			=> getCurrentTime(),
			'on_insert_admin_id' 	=> $this->session->get('id'),
            'on_active'          	=> 1
        ];

        $outgoingNoteModel = new OutgoingNoteModel();
        $outgoingNoteDetailModel = new OutgoingNoteDetailModel();
		$onID = $outgoingNoteModel->addOutgoingNote($data);

		$ongoingDetail = [];
		foreach ($bitQuantity as $key => $value) {

			if (@$bitQuantity[$key] != '' && @$bitWeight[$key] != '') {

				$ongoingDetail[] = [
					'ond_on_id' => $onID,
					'ond_bi_id' => @$batchItemTaskData[0]->bi_id,
					'ond_bit_id' => $bitID,
					'ond_name' => @$bitName[$key],
					'ond_quantity' => @$bitQuantity[$key],
					'ond_weight' => @$bitWeight[$key],
					'ond_insert_at' => getCurrentTime(),
					'ond_insert_admin_id' => $this->session->get('id'),
				];
			}
		}

		$ondID = $outgoingNoteDetailModel->addOutgoingNoteDetailBulk($ongoingDetail);

		if ($onID && $ondID) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'modal_close' => '#kt_modal_outgoing_note',
			];

			$outgoingNoteDetailModel = new OutgoingNoteDetailModel();
			$outgoingNoteDetailData = $outgoingNoteDetailModel->getCustom([
				'select' => ' sum(ond_weight) as sum_total_weight, sum(ond_quantity) as "sum_total_quantity" ',
				'where' => ' AND on_types="Bahan Sisa" AND ond_bit_id="'.$bitID.'" AND on_active=1 ',
				'groupby' => '-'
			]);

			$data['sum_outgoing_note_weight'] = $outgoingNoteDetailData[0]->sum_total_weight;
			$data['sum_outgoing_note_quantity'] = $outgoingNoteDetailData[0]->sum_total_quantity;

			return json_encode($data);
		}
		
	}

	public function print($id){

		$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteDetailModel = new OutgoingNoteDetailModel();

		$outgoingNoteData = $outgoingNoteModel->getAll([
			'where' => ' AND on_id="'.$id.'" ',
			'orderby'=> 'on_id DESC'
		]);


		$outgoingNoteDetailData = $outgoingNoteDetailModel->getAll([
			'where' => ' AND ond_on_id="'.$id.'" ',
			'orderby'=> ' ond_id ASC'
		]);

		$data = [
			'title' => lang('Menu.order.outgoing-note.main'),
			'menu' => 'outgoing-note',
			'data' => $outgoingNoteData[0],
			'data_item' => $outgoingNoteDetailData
		];

		return view('outgoing-note/print', $data);

	}


	public function deleteOutgoingNote($id)
	{

		$outgoingNoteDetailModel = new OutgoingNoteDetailModel();
		$outgoingNoteModel = new OutgoingNoteModel();

		$outgoingNoteData = $outgoingNoteModel->deleteOutgoingNote($id, $this->session->get('id'));

		if ($outgoingNoteData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}else{
			$data = [
				'error'=> lang('Validation.error_delete'),
			];

			return json_encode($data);
		}
	}


	public function noteOutgoingNote($date){
    	$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteNota = $outgoingNoteModel->setNota();

		if ($outgoingNoteNota) {
			$number = $outgoingNoteNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'ON/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
    }


    public function noteOutgoingNoteProcess($date){
    	$outgoingNoteModel = new OutgoingNoteModel();
		$outgoingNoteNota = $outgoingNoteModel->setNota();

		if ($outgoingNoteNota) {
			$number = $outgoingNoteNota[0]->total;
	        $number = ((int)$number) + 1;
	        $code = str_pad($number, 5, "0", STR_PAD_LEFT);
		}else{
			$code = '00001';
		}

		$dateEx = explode('-', $date);

		return 'ONP/'.$dateEx[2].$dateEx[1].substr($dateEx[0], -2) .'/'.$code;
    }
	
}