<?php

namespace App\Controllers;

use App\Models\SizeModel;
use App\Models\ItemModel;

class Size extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.size.main'),
			'menu' => 'size',
		];

		return view('size/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$sizeModel = new SizeModel();

		$sizeData = $sizeModel->getAll(); 

		$data = [];

		if (@$sizeData) {
        	foreach ($sizeData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/size/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/size/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }


	public function addSize()
	{
		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.size.main'),
				'menu' => 'size',
			];

			return view('size/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Ukuran', 'rules' => 'required|max_length[255]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$sizeModel = new SizeModel();

		$sizeData = $sizeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($sizeData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }

		$data = [
			'size_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'size_name' 		     => $this->request->getPost('name'),
            'size_insert_admin_id' => $this->session->get('id'),
            'size_active'          => 1
        ];


		$sizeData = $sizeModel->addSize($data); 
		

		if ($sizeData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'size'
			];

			return json_encode($data);
		}

	}

	public function updateSize($id)
	{
		$sizeModel = new SizeModel();
		$sizeData = $sizeModel->getByID($id);

		if (!$this->request->isAjax()) {
			$data = [
				'data' => (isset($sizeData))? $sizeData : [],
				'title' => lang('Menu.user.size.main'),
				'menu' => 'size',
			];

			return view('size/edit',  $data);
		}

		$data = [
			'size_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'size_name' 		     => $this->request->getPost('name'),
            'size_update_admin_id' => $this->session->get('id'),
            'size_active'          => 1
        ];

		$rules = [];
        $sizeData = $sizeModel->getByID($id);
        if ($sizeData->name != $data['size_name']) {
        	$sizeData = $sizeModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($sizeData) > 0) {
					$msg = [
		                'error' => lang('Validation.name_already_exist'),
		            ];
		            return json_encode($msg);
	        }
        }


    	$rules['name'] = ['label' => 'Nama Ukuran', 'rules' => 'required|max_length[255]'];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}
		$sizeData = $sizeModel->updateSize($data, $id); 

		if ($sizeData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'size'
			];

			return json_encode($data);
		}
	}

	public function deleteSize($id)
	{

		$itemModel = new ItemModel();
		if(count($itemModel->getAll(["where" => ' AND item_size_id="'.$id.'" AND item_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.size_already_use'),
			];
			return json_encode($data);
		}

		$sizeModel = new SizeModel();

		$sizeData = $sizeModel->deleteSize($id, $this->session->get('id')); 

		if ($sizeData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}