<?php

namespace App\Controllers;

use App\Models\WorkflowModel;
use App\Models\ItemModel;
use App\Models\BatchModel;
use App\Models\TaskModel;

class Workflow extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.workflow.main'),
			'menu' => 'workflow',
		];

		return view('workflow/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$workflowModel = new WorkflowModel();

		$workflowData = $workflowModel->getAll(); 

		$data = [];

		if (@$workflowData) {
        	foreach ($workflowData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/workflow/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/workflow/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }


	public function addWorkflow()
	{
		$taskModel = new TaskModel();

		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.workflow.main'),
				'data_task' => $taskModel->getAll(),
				'menu' => 'workflow',
			];

			return view('workflow/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Alur Kerja', 'rules' => 'required|max_length[255]'],
            'task_id.0'             => ['label' => 'Divisi', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'workflow_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'workflow_name' 		     => $this->request->getPost('name'),
            'workflow_insert_admin_id' => $this->session->get('id'),
            'workflow_active'          => 1
        ];

		$workflowModel = new WorkflowModel();

		$workflowData = $workflowModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($workflowData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }

        $data2 = [];

		$workflowData = $workflowModel->addWorkflow($data); 
        foreach ($this->request->getPost('task_id') as $key => $value) {
        	$data2[] = [
        		'wt_workflow_id'=> $workflowData,
        		'wt_task_id'=> $value,
        		'wt_order'=> $key+1,
        		'wt_insert_admin_id'=> $this->session->get('id'),
        		'wt_active' => 1
        	];
        }

		$workflowTaskData = $workflowModel->addWorkflowTaskBluk($data2); 

		if ($workflowData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect'=> 'workflow'
			];

			return json_encode($data);
		}

	}

	public function updateWorkflow($id)
	{
		$workflowModel = new WorkflowModel();

		$workflowData = $workflowModel->getByID($id);

		if (!$this->request->isAjax()) {

			$taskModel = new TaskModel();

			$data = [
				'data' => (isset($workflowData))? $workflowData : [],
				'title' => lang('Menu.master.workflow.main'),
				'menu' => 'workflow',
				'data_task' => $taskModel->getAll(),
				'data_workflow_task'=> $workflowModel->getWorkflowTaskGetByWorkflow($id),
			];

			return view('workflow/edit',  $data);
		}

		$data = [
			'workflow_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'workflow_name' 		     => $this->request->getPost('name'),
            'workflow_update_admin_id' => $this->session->get('id'),
            'workflow_active'          => 1
        ];

		if (!$this->validate([
            'name'             => ['label' => 'Nama Alur Kerja', 'rules' => 'required|max_length[255]'],
            'task_id.0'             => ['label' => 'Divisi', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		if ($workflowData->name != $data['workflow_name']) {
        	$workflowDataByCode = $workflowModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($workflowDataByCode) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
	        }
        }


		$workflowData = $workflowModel->updateWorkflow($data, $id); 

		$data2 = [];
		foreach ($this->request->getPost('task_id') as $key => $value) {
        	$data2[] = [
        		'wt_workflow_id'=> $id,
        		'wt_task_id'=> $value,
        		'wt_order'=> $key+1,
        		'wt_insert_admin_id'=> $this->session->get('id'),
        		'wt_active' => 1
        	];
        }

        $workflowModel->deleteWorkflowTask($id, $this->session->get('id'));

		$workflowTaskData = $workflowModel->addWorkflowTaskBluk($data2); 

		if ($workflowData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'workflow'
			];

			return json_encode($data);
		}
	}

	public function deleteWorkflow($id)
	{
		$itemModel = new ItemModel();
		if(count($itemModel->getAll(["where" => ' AND item_workflow_id="'.$id.'" AND item_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.workflow_already_use'),
			];
			return json_encode($data);
		}

		$workflowModel = new WorkflowModel();
		if(count($workflowModel->getWorkflowTaskGetByWorkflow($workflowID))>0){
			$data = [
				'error'=> lang('Validation.workflow_already_use_task'),
			];
			return json_encode($data);
		}

		$batchModel = new BatchModel();
		if(count($batchModel->getAll(["where" => ' AND batch_workflow_id="'.$id.'" AND batch_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.workflow_already_use_batch'),
			];
			return json_encode($data);
		}


		$workflowData = $workflowModel->deleteWorkflow($id, $this->session->get('id')); 

		if ($workflowData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}