<?php

namespace Config;

// Create a new instance of our RouteCollection class.
$routes = Services::routes();

// Load the system's routing file first, so that the app and ENVIRONMENT
// can override as needed.
if (file_exists(SYSTEMPATH . 'Config/Routes.php'))
{
	require SYSTEMPATH . 'Config/Routes.php';
}

/**
 * --------------------------------------------------------------------
 * Router Setup
 * --------------------------------------------------------------------
 */
$routes->setDefaultNamespace('App\Controllers');
$routes->setDefaultController('Auth');
$routes->setDefaultMethod('index');
$routes->setTranslateURIDashes(false);
$routes->set404Override();
$routes->setAutoRoute(true);

/*
 * --------------------------------------------------------------------
 * Route Definitions
 * --------------------------------------------------------------------
 */

// We get a performance increase by specifying the default
// route since we don't have to scan directories.
$routes->get('/', 'AdminLogin::index');
$routes->get('/auth', 'AdminLogin::index');
$routes->post('/auth', 'AdminLogin::login_check');
$routes->get('/auth/logout', 'AdminLogin::logout');
$routes->get('/admin/edit-password', 'Admin::editPassword');
$routes->post('/admin/edit-password', 'Admin::editPassword');

$routes->get('lang/{locale}', 'Language::index');
$routes->get('/dashboard', 'Dashboard::index');
$routes->get('/dashboard/order-note', 'Dashboard::orderNote');
$routes->get('/dashboard/order-item-detail/(:segment)', 'Dashboard::orderItemDetail/$1');

$routes->get('/admin', 'Admin::getAll');
$routes->get('/admin/data', 'Admin::data');
$routes->get('/admin/create', 'Admin::addAdmin');
$routes->post('/admin/create', 'Admin::addAdmin');
$routes->get('/admin/edit/(:segment)', 'Admin::updateAdmin/$1');
$routes->post('/admin/edit/(:segment)', 'Admin::updateAdmin/$1');
$routes->post('/admin/delete/(:segment)', 'Admin::deleteAdmin/$1');

$routes->get('/admin/role', 'AdminRole::getAll');
$routes->get('/admin/role/data', 'AdminRole::data');
$routes->get('/admin/role/create', 'AdminRole::addAdminRole');
$routes->post('/admin/role/create', 'AdminRole::addAdminRole');
$routes->get('/admin/role/edit/(:segment)', 'AdminRole::updateAdminRole/$1');
$routes->post('/admin/role/edit/(:segment)', 'AdminRole::updateAdminRole/$1');
$routes->post('/admin/role/delete/(:segment)', 'AdminRole::deleteAdminRole/$1');

$routes->get('/admin/permission/create/(:segment)', 'AdminPermission::updateAdminPermission/$1');
$routes->post('/admin/permission/create/(:segment)', 'AdminPermission::updateAdminPermission/$1');

$routes->get('/admin/module/create/(:segment)', 'AdminModule::addAdminModule/$1');
$routes->post('/admin/module/create/(:segment)', 'AdminModule::addAdminModule/$1');
$routes->post('/admin/module/update/(:segment)', 'AdminModule::updateAdminModule/$1');

$routes->get('/tree', 'Tree::getAll');
$routes->get('/tree/data', 'Tree::data');
$routes->get('/tree/create', 'Tree::addTree');
$routes->post('/tree/create', 'Tree::addTree');
$routes->get('/tree/edit/(:segment)', 'Tree::updateTree/$1');
$routes->post('/tree/edit/(:segment)', 'Tree::updateTree/$1');
$routes->post('/tree/delete/(:segment)', 'Tree::deleteTree/$1');

$routes->get('/gold-content', 'GoldContent::getAll');
$routes->get('/gold-content/data', 'GoldContent::data');
$routes->get('/gold-content/create', 'GoldContent::addGoldContent');
$routes->post('/gold-content/create', 'GoldContent::addGoldContent');
$routes->get('/gold-content/edit/(:segment)', 'GoldContent::updateGoldContent/$1');
$routes->post('/gold-content/edit/(:segment)', 'GoldContent::updateGoldContent/$1');
$routes->post('/gold-content/delete/(:segment)', 'GoldContent::deleteGoldContent/$1');

$routes->get('/gold-type', 'GoldType::getAll');
$routes->get('/gold-type/data', 'GoldType::data');
$routes->get('/gold-type/create', 'GoldType::addGoldType');
$routes->post('/gold-type/create', 'GoldType::addGoldType');
$routes->get('/gold-type/edit/(:segment)', 'GoldType::updateGoldType/$1');
$routes->post('/gold-type/edit/(:segment)', 'GoldType::updateGoldType/$1');
$routes->post('/gold-type/delete/(:segment)', 'GoldType::deleteGoldType/$1');

$routes->get('/size', 'Size::getAll');
$routes->get('/size/data', 'Size::data');
$routes->get('/size/create', 'Size::addSize');
$routes->post('/size/create', 'Size::addSize');
$routes->get('/size/edit/(:segment)', 'Size::updateSize/$1');
$routes->post('/size/edit/(:segment)', 'Size::updateSize/$1');
$routes->post('/size/delete/(:segment)', 'Size::deleteSize/$1');

$routes->get('/item', 'Item::getAll');
$routes->get('/item/data', 'Item::data');
$routes->get('/item/create', 'Item::addItem');
$routes->post('/item/create', 'Item::addItem');
$routes->get('/item/edit/(:segment)', 'Item::updateItem/$1');
$routes->post('/item/edit/(:segment)', 'Item::updateItem/$1');
$routes->post('/item/delete/(:segment)', 'Item::deleteItem/$1');

$routes->get('/item/material', 'ItemMaterial::getAll');
$routes->get('/item/material/data', 'ItemMaterial::data');
$routes->get('/item/material/create', 'ItemMaterial::addItem');
$routes->post('/item/material/create', 'ItemMaterial::addItem');
$routes->get('/item/material/edit/(:segment)', 'ItemMaterial::updateItem/$1');
$routes->post('/item/material/edit/(:segment)', 'ItemMaterial::updateItem/$1');
$routes->post('/item/material/delete/(:segment)', 'ItemMaterial::deleteItem/$1');

$routes->get('/category', 'Category::getAll');
$routes->get('/category/data', 'Category::data');
$routes->get('/category/create', 'Category::addCategory');
$routes->post('/category/create', 'Category::addCategory');
$routes->get('/category/edit/(:segment)', 'Category::updateCategory/$1');
$routes->post('/category/edit/(:segment)', 'Category::updateCategory/$1');
$routes->post('/category/delete/(:segment)', 'Category::deleteCategory/$1');

$routes->get('/workflow', 'Workflow::getAll');
$routes->get('/workflow/data', 'Workflow::data');
$routes->get('/workflow/create', 'Workflow::addWorkflow');
$routes->post('/workflow/create', 'Workflow::addWorkflow');
$routes->get('/workflow/edit/(:segment)', 'Workflow::updateWorkflow/$1');
$routes->post('/workflow/edit/(:segment)', 'Workflow::updateWorkflow/$1');
$routes->post('/workflow/delete/(:segment)', 'Workflow::deleteWorkflow/$1');

$routes->get('/order', 'Order::getAll');
$routes->get('/order/data', 'Order::data');
$routes->get('/order/note/(:segment)', 'Order::note/$1');
$routes->get('/order/create', 'Order::addOrder');
$routes->post('/order/create', 'Order::addOrder');
$routes->get('/order/edit/(:segment)', 'Order::updateOrder/$1');
$routes->post('/order/edit/(:segment)', 'Order::updateOrder/$1');
$routes->post('/order/delete/(:segment)', 'Order::deleteOrder/$1');
$routes->post('/order/add-item-order', 'Order::addItemOrder/$1');

$routes->get('/customer', 'Customer::getAll');
$routes->get('/customer/data', 'Customer::data');
$routes->get('/customer/create', 'Customer::addCustomer');
$routes->post('/customer/create', 'Customer::addCustomer');
$routes->get('/customer/edit/(:segment)', 'Customer::updateCustomer/$1');
$routes->post('/customer/edit/(:segment)', 'Customer::updateCustomer/$1');
$routes->post('/customer/delete/(:segment)', 'Customer::deleteCustomer/$1');

$routes->get('/order-item', 'OrderItem::getAll');
$routes->get('/order-item/data', 'OrderItem::data');
$routes->post('/order-item/edit', 'OrderItem::updateOrderItem');
$routes->post('/order-item/delete/(:segment)', 'OrderItem::deleteOrderItem/$1');

$routes->get('/batch', 'Batch::getAll');
$routes->get('/batch/data', 'Batch::data');
$routes->get('/batch/create', 'Batch::addBatch');
$routes->get('/batch/order-item-data', 'Batch::orderItemData');
$routes->post('/batch/create', 'Batch::addBatch');
$routes->get('/batch/edit/(:segment)', 'Batch::updateBatch/$1');
$routes->get('/batch/note/(:segment)', 'Batch::note/$1');
$routes->post('/batch/edit/(:segment)', 'Batch::updateBatch/$1');
$routes->post('/batch/delete/(:segment)', 'Batch::deleteBatch/$1');
$routes->get('/batch/item-task/(:segment)', 'Batch::printTask/$1');

$routes->post('/batch-item-task/create/(:segment)', 'BatchItemTask::store/$1');
$routes->get('/batch-item-task', 'BatchItemTask::index');
$routes->get('/batch-item-task/data', 'BatchItemTask::data');
$routes->get('/batch-item-task/(:segment)', 'BatchItemTask::edit/$1');
$routes->post('/batch-item-task/(:segment)', 'BatchItemTask::edit/$1');
$routes->post('/batch-item-task/add', 'Batch::addItemBatch/$1');
$routes->get('/batch-item-task/print/(:segment)', 'BatchItemTask::print/$1');
$routes->get('/batch-item-task/edit/(:segment)', 'BatchItemTask::editTask/$1');
$routes->post('/batch-item-task/edit/(:segment)', 'BatchItemTask::editTask/$1');
$routes->get('/worker/rapair-output', 'Worker::repairBit');

$routes->post('/batch-item-request/create', 'BatchItemRequest::store');
$routes->get('/batch-item-request/(:segment)', 'BatchItemRequest::data/$1');
$routes->get('/batch-item-request/print/(:segment)', 'BatchItemRequest::print/$1');
$routes->post('/batch-item-request/(:segment)', 'BatchItemRequest::update/$1');
$routes->post('/batch-item-request/delete/(:segment)', 'BatchItemRequest::delete/$1');
$routes->post('/worker/batch-item-request/approve', 'Worker::approveBatchItemRequest');
$routes->post('/worker/batch-item-request/cencel', 'Worker::cencelBatchItemRequest');

$routes->get('/batch-item-output', 'BatchItemOutput::index');
$routes->post('/batch-item-output/create', 'BatchItemOutput::store');

$routes->get('/task', 'Task::getAll');
$routes->get('/task/data', 'Task::data');
$routes->get('/task/create', 'Task::addTask');
$routes->post('/task/create', 'Task::addTask');
$routes->get('/task/edit/(:segment)', 'Task::updateTask/$1');
$routes->post('/task/edit/(:segment)', 'Task::updateTask/$1');
$routes->post('/task/delete/(:segment)', 'Task::deleteTask/$1');

$routes->get('/report/order', 'Report::order');
$routes->get('/report/export-order', 'Report::exportOrder');
$routes->get('/report/batch', 'Report::batch');
$routes->get('/report/export-batch', 'Report::exportBatch');
$routes->get('/report/batch-item-task', 'BatchItemTask::report');

$routes->get('/report/outgoing-note', 'Report::outgoingNote');
$routes->get('/report/export-outgoing-note', 'Report::exportOutgoingNote');

$routes->get('/report/outgoing-batch', 'Report::outgoingBatch');
$routes->get('/report/export-outgoing-batch', 'Report::exportOutgoingBatch');

$routes->get('/report/production', 'Report::production');
$routes->get('/report/export-production', 'Report::exportProduction');

$routes->get('/outgoing-note', 'OutgoingNote::getAll');
$routes->post('/outgoing-note', 'OutgoingNote::getAll');
$routes->get('/outgoing-note/data', 'OutgoingNote::data');
$routes->get('/outgoing-note/create', 'OutgoingNote::addOutgoingNote');
$routes->post('/outgoing-note/create', 'OutgoingNote::addOutgoingNote');
$routes->get('/outgoing-note/batch-item-data', 'OutgoingNote::batchItemData');
$routes->post('/outgoing-note/delete/(:segment)', 'OutgoingNote::deleteOutgoingNote/$1');
$routes->get('/outgoing-note/print/(:segment)', 'OutgoingNote::print/$1');

$routes->get('/outgoing-batch', 'OutgoingBatch::getAll');
$routes->post('/outgoing-batch', 'OutgoingBatch::getAll');
$routes->get('/outgoing-batch/data', 'OutgoingBatch::data');
$routes->get('/outgoing-batch/create', 'OutgoingBatch::addOutgoingBatch');
$routes->post('/outgoing-batch/create', 'OutgoingBatch::addOutgoingBatch');
$routes->get('/outgoing-batch/batch-item-data', 'OutgoingBatch::batchItemData');
$routes->post('/outgoing-batch/delete/(:segment)', 'OutgoingBatch::deleteOutgoingBatch/$1');
$routes->get('/outgoing-batch/print/(:segment)', 'OutgoingBatch::print/$1');

$routes->post('/worker/tester', 'Worker::tester');

/*
 * --------------------------------------------------------------------
 * Additional Routing
 * --------------------------------------------------------------------
 *
 * There will often be times that you need additional routing and you
 * need it to be able to override any defaults in this file. Environment
 * based routes is one such time. require() additional route files here
 * to make that happen.
 *
 * You will have access to the $routes object within that file without
 * needing to reload it.
 */
if (file_exists(APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php'))
{
	require APPPATH . 'Config/' . ENVIRONMENT . '/Routes.php';
}
