<?php

namespace App\Controllers;

use CodeIgniter\Controller;
use CodeIgniter\HTTP\CLIRequest;
use CodeIgniter\HTTP\IncomingRequest;
use CodeIgniter\HTTP\RequestInterface;
use CodeIgniter\HTTP\ResponseInterface;
use Psr\Log\LoggerInterface;

use App\Models\AdminModel;
use App\Models\AdminRoleModuleModel;
use App\Models\AdminPermissionModel;

/**
 * Class BaseController
 *
 * BaseController provides a convenient place for loading components
 * and performing functions that are needed by all your controllers.
 * Extend this class in any new controllers:
 *     class Home extends BaseController
 *
 * For security be sure to declare any new methods as protected or private.
 */

class BaseController extends Controller
{
	/**
	 * Instance of the main Request object.
	 *
	 * @var IncomingRequest|CLIRequest
	 */

	/**
	 * An array of helpers to be loaded automatically upon
	 * class instantiation. These helpers will be available
	 * to all other controllers that extend BaseController.
	 *
	 * @var array
	 */
	protected $helpers = [];
	protected $response;
	protected $request;
	protected $benchmark;
	protected $accessScope;
	protected $accessToken;
	protected $sessionData;
	protected $session;
	protected $sessionAdminData;
	protected $sessionStudentData;
	protected $validation;
	protected $cache;
	protected $cacheTTL;
	protected $class;
	protected $method;
	protected $language;

	/**
	 * Constructor.
	 *
	 * @param RequestInterface  $request
	 * @param ResponseInterface $response
	 * @param LoggerInterface   $logger
	 */
	public function initController(RequestInterface $request, ResponseInterface $response, LoggerInterface $logger)
	{
		// Do Not Edit This Line
		parent::initController($request, $response, $logger);
	}

	public function __construct(){
		$this->benchmark = \Config\Services::timer();
		$this->benchmark->start('render view');
		helper('app');
		helper('form');
		helper('file');

		$throttler = \Config\Services::throttler();
		$requestLimit = $throttler->check('skpnk_sms', 1000, MINUTE);
		if(!$requestLimit){
			return getResponse(429, "error", ['error_message' => lang('ErrorMessages.tooManyRequest')], $this->benchmark);
		}


		$this->response = \Config\Services::response();
		$this->request = \Config\Services::request();
		$this->validation =  \Config\Services::validation();
		$this->session = \Config\Services::session();
		$this->cache = \Config\Services::cache();
		$this->cacheTTL = 18000;
		$this->cacheTTLVideo = 3600;

		$this->language = \Config\Services::language();
		$this->language->setLocale('en');

		$router = service('router');
		$this->class  = $router->controllerName();
		$this->method = $router->methodName();

		$sessionDataID =  $this->session->get('id');

		if($this->_checkAdminAccess() == false){
			echo json_encode([
				'error' => lang('Validation.error_access_forbidden'),
				'redirect' => 'auth'
			]);
			exit();
		}
	}

	private function _checkAdminAccess(){
		$accessGranted = false;
		$generalController = array(
			'\App\Controllers\AdminLogin',
			'\App\Controllers\Worker',
		);

		$generalControllerLogin = array(
			'\App\Controllers\Dashboard',
		);

		if (in_array($this->class, $generalController)) {
			return true;
		}

		if(!in_array($this->class, $generalController)){

			if(@$this->session->get('id')){

				if (in_array($this->class, $generalControllerLogin)) {
					return true;
				}

				$adminRoleModuleModel = new AdminRoleModuleModel();
				$adminPermissionModel = new AdminPermissionModel();
		
				$adminPermissionData = $adminPermissionModel->getByAdminID($this->session->get('id')); 
				foreach($adminPermissionData as $permission){
					$adminRoleModuleData = $adminRoleModuleModel->getByRoleID($permission->role_id);
					foreach($adminRoleModuleData as $roleModule){
						if($roleModule->class == $this->class && $roleModule->method == $this->method){
							$accessGranted = TRUE;
							break;
						}
					}

					if($accessGranted){
						break;
					}
				}
			}
		}
		//return true;
		return $accessGranted;
    }


	public function filterArray($array, $allowed = [])
    {
        return array_filter(
            $array,
            function ($val, $key) use ($allowed) { 
                return isset($allowed[$key]) && ($allowed[$key] === true || $allowed[$key] === $val);
            },
        );
    }

    public function filterKeyword($data, $search, $field = '')
    {
        $filter = '';
        if (isset($search['value'])) {
            $filter = $search['value'];
        }
        if (!empty($filter)) {
            if (!empty($field)) {
                if (strpos(strtolower($field), 'date') !== false) {
                    // filter by date range
                    $data = $this->filterByDateRange($data, $filter, $field);
                } else {
                    // filter by column
                    $data = array_filter($data, function ($a) use ($field, $filter) {
                        return (boolean) preg_match("/$filter/i", $a[$field]);
                    });
                }

            } else {
                // general filter
                $data = array_filter($data, function ($a) use ($filter) {
                    return (boolean) preg_grep("/$filter/i", (array) $a);
                });
            }
        }

        return $data;
    }

    public function filterByDateRange($data, $filter, $field)
    {
        // filter by range
        if (!empty($range = array_filter(explode('|', $filter)))) {
            $filter = $range;
        }

        if (is_array($filter)) {
            foreach ($filter as &$date) {
                // hardcoded date format
                $date = date_create_from_format('m/d/Y', stripcslashes($date));
            }
            // filter by date range
            $data = array_filter($data, function ($a) use ($field, $filter) {
                // hardcoded date format
                $current = date_create_from_format('m/d/Y', $a[$field]);
                $from    = $filter[0];
                $to      = $filter[1];
                if ($from <= $current && $to >= $current) {
                    return true;
                }

                return false;
            });
        }

        return $data;
    }

    public function arraySearch($array, $keyword)
    {

		$keyword = filter_var($keyword, FILTER_SANITIZE_STRING);

		$keyword = trim($keyword);

        return array_filter($array, function ($a) use ($keyword) {
            return (boolean) preg_grep("~".$keyword."~", (array) $a);
        });
    }
}