<?php

namespace App\Controllers;

use App\Models\ItemMaterialModel;
use App\Models\ItemModel;

class ItemMaterial extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}

		$itemID = $this->request->getVar('item_id');
		
		$data = [
			'title' => lang('Menu.master.item-material.main'),
			'menu' => 'item-material',
		];

		return view('item-material/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}

		$itemID = $this->request->getVar('item_id');
		
		$ItemMaterialModel = new ItemMaterialModel();

		$param = [];

		if ($itemID != '') {
			$param['where'] = ' AND im_item_id="'.$itemID.'" ';
		}

		$ItemMaterialData = $ItemMaterialModel->getAll($param);

		$data = [];

		if (@$ItemMaterialData) {
        	foreach ($ItemMaterialData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/item-material/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/item-material/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['source'] = $val->code;
            $value['required_to_check'] = $val->required_to_check;
            $value['name'] = $val->name;
            $value['quantity'] = $val->quantity;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addItemMaterial()
	{
		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.item-material.main'),
				'menu' => 'item-material',
			];

			return view('item-material/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama kategori ', 'rules' => 'required|max_length[255]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'im_source'=> $this->request->getPost('source'),
			'im_required_to_check' => $this->request->getPost('required_to_check'),
			'im_item_id'=> $this->request->getVar('item_id'),
			'im_name'=> $this->request->getPost('name'),
			'im_quantity'=> $this->request->getPost('quantity'),
			'im_insert_at'=> date('Y-m-d H:i:s'),
			'im_insert_admin_id'=> $this->session->get('id'),
			'im_update_at' => date('Y-m-d H:i:s'),
			'im_update_admin_id' => $this->session->get('id'),	
			'im_active' => 1,
        ];

		$ItemMaterialModel = new ItemMaterialModel();

		$ItemMaterialData = $ItemMaterialModel->addItemMaterial($data); 
		

		if ($ItemMaterialData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect'=> 'item-material'
			];

			return json_encode($data);
		}

	}

	public function updateItemMaterial()
	{
		$ItemMaterialModel = new ItemMaterialModel();

		$ItemMaterialData = $ItemMaterialModel->getByID($id);

		if (!$this->request->isAjax()) {
			$data = [
				'data' => (isset($ItemMaterialData))? $ItemMaterialData : [],
				'title' => lang('Menu.user.item-material.main'),
				'menu' => 'item-material',
			];

			return view('item-material/edit',  $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Kategori', 'rules' => 'required|max_length[255]'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}


		$data = [
			'im_source'=> $this->request->getPost('source'),
			'im_required_to_check' => $this->request->getPost('required_to_check'),
			'im_item_id'=> $this->request->getVar('item_id'),
			'im_name'=> $this->request->getPost('name'),
			'im_quantity'=> $this->request->getPost('quantity'),
			'im_update_at' => date('Y-m-d H:i:s'),
			'im_update_admin_id' => $this->session->get('id'),	
			'im_active' => 1,
        ];

		$ItemMaterialModel = new ItemMaterialModel();

		$ItemMaterialData = $ItemMaterialModel->updateItemMaterial($data, $id); 

		if ($ItemMaterialData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'item-material'
			];

			return json_encode($data);
		}
	}

	public function deleteItemMaterial($id)
	{
		$itemModel = new ItemModel();
		if(count($itemModel->getAll(["where" => ' AND im_id="'.$id.'" AND item_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.im_already_use'),
			];
			return json_encode($data);
		}

		$ItemMaterialModel = new ItemMaterialModel();

		$ItemMaterialData = $ItemMaterialModel->deleteItemMaterial($id, $this->session->get('id')); 

		if ($ItemMaterialData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}
}