<?php
namespace App\Controllers;

use App\Models\ItemModel;
use App\Models\OrderModel;
use App\Models\OrderItemModel;
use App\Models\WorkflowModel;
use App\Models\TaskModel;
use App\Models\BatchModel;
use App\Models\BatchItemModel;
use Ramsey\Uuid\Uuid;
use App\Libraries\S3Library;
use App\Libraries\Barcode;
use App\Models\AdminRoleModuleModel;
use App\Models\TreeModel;
use App\Models\PrintHistoryModel;
use App\Models\BatchItemTaskModel;
use App\Models\BatchItemRequestModel;
use App\Models\BatchItemOutputModel;

class BatchItemTask extends BaseController
{
	public function index()
	{
		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
		];

		return view('batch-item-task/index', $data);
	}

	public function store($id){

		$batchModel = new BatchModel();

		$batchItemTaskModel = new BatchItemTaskModel();

		$batchData = $batchModel->getByID($id);

		$batchItemModel = new BatchItemModel();
		$workflowModel = new WorkflowModel();

		$batchItemData = $batchItemModel->getByBatchID($id);
		$taskModel = new TaskModel();

		$batchItemRequestModel = new BatchItemRequestModel();
		
		$data = [];

		foreach ($batchItemData as $key => $value) {
			$s3Library = new S3Library();
			$batchItemRequestGet = $batchItemRequestModel->getAll(['where' => 'AND bir_status="New" AND bir_bi_id="'.$value->id.'" AND bir_active=1 ']);

			if(count($batchItemRequestGet) > 0){
				$msg = [
	                'error' => lang('ErrorMessages.materialNotYetApproved'),
	            ];
	            return json_encode($msg);
			}
		}

		if($batchData->total_weight == 0 || $batchData->total_gold_weight == 0){
			$msg = [
                'error' => lang('ErrorMessages.batchIncomplete'),
            ];
            return json_encode($msg);
		}

		foreach ($batchItemData as $key => $value) {
			$workflowID = $value->workflow_id;

			$workflowTaskData = $workflowModel->getWorkflowTaskGetByWorkflow($workflowID);

			$arrayBatchID = [];
			foreach ($batchItemData as $key_ => $value_) {
				$arrayBatchID[] = $value_->id;
			}

			if (empty($arrayBatchID)) {
				return false;
			}

			$checkBatchItemTask = $batchItemTaskModel->getAll(['where' => ' AND bit_bi_id IN ('.implode(',', $arrayBatchID).') ']);

			if (!empty($checkBatchItemTask)) {
				return true;
			}

			$groupNote = $this->groupNote(); 

			foreach ($workflowTaskData as $key1 => $value1) {

				$taskInputData = [];
				$taskResult = $taskModel->taskInputFieldGetByTask($value1->task_id, 'input');
				foreach ($taskResult as $key2 => $value2) {
					$taskInputData[$key2]['field_name'] = $value2->name;
					$taskInputData[$key2]['is_required'] = $value2->is_required;
					$taskInputData[$key2]['datatype'] = $value2->datatype;
					$taskInputData[$key2]['options'] = $value2->options;
					$taskInputData[$key2]['answer'] = '';
				}

				$taskOutputData = [];
				$taskResult3 = $taskModel->taskInputFieldGetByTask($value1->task_id, 'output');
				foreach ($taskResult3 as $key3 => $value3) {
					$taskOutputData[$key3]['field_name'] = $value3->name;
					$taskOutputData[$key3]['is_required'] = $value3->is_required;
					$taskOutputData[$key3]['datatype'] = $value3->datatype;
					$taskOutputData[$key3]['options'] = $value3->options;
					$taskOutputData[$key3]['answer'] = '';    
				}

				$data[] = [
					'bit_bi_id' => $value->id,
					'bit_order' => ($key1+1),
					'bit_group_note' => $groupNote,
					'bit_workflow_id' => $value1->workflow_id,
					'bit_task_id' => $value1->task_id,
					'bit_input_operator_name' => '',
					'bit_output_operator_name' => '',
					'bit_input_fields' => json_encode($taskInputData),
					'bit_output_fields' => json_encode($taskOutputData),
					'bit_input_quantity' => 0,
					'bit_output_quantity' => 0,
					'bit_input_weight' => 0,
					'bit_output_weight' => 0,
					'bit_status_field' => ($key1 == 0)? 'input' : '',
					'bit_status' => ($key1 == 0)? 'Proses' : 'New',
					'bit_insert_at' => getCurrentTime(),
					'bit_insert_admin_id' => $this->session->get('id')
				];
			}
		}


		$data2['batch_status'] = 'Proses';
		$batchID = $batchModel->updateBatches($data2, $id);

		$result = $batchItemTaskModel->addBatchItemTaskBulk($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => ''
			];

			return json_encode($data);
		}
	}

	public function store_item_request($id){
		

		// bir_id bir_bi_id bir_bit_id	bir_item_sku bir_item_name bir_item_quantity	bir_item_weight

		$rules = [
            'item_sku' => ['label' => 'SKU', 'rules' => 'required|max_length[255]'],
            'item_name' => ['label' => 'Nama', 'rules' => 'required|max_length[255]'],
            'item_quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
            'item_weight' => ['label' => 'Weight', 'rules' => 'required|max_length[65]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'bir_bi_id' => $id,
			'bir_bit_id' => $this->request->getPost('bit_id', FILTER_SANITIZE_STRING),
			'bir_item_sku' => $this->request->getPost('item_sku', FILTER_SANITIZE_STRING),
			'bir_item_name' => $this->request->getPost('item_name', FILTER_SANITIZE_STRING),
			'bir_item_quantity' => $this->request->getPost('item_quantity', FILTER_SANITIZE_STRING),
			'bir_item_weight' => $this->request->getPost('item_weight', FILTER_SANITIZE_STRING),
			'bir_insert_at' => getCurrentTime(),
			'bir_insert_admin_id' => $this->session->get('id'),
			'bir_active' => 1
		];

		$batchItemRequestModel = new BatchItemRequestModel();
		$result = $batchItemRequestModel->addBatchItemRequest($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'reload' => true,
			];

			return json_encode($data);
		}
	}

	public function data(){
		// if (!$this->request->isAjax()) {
		// 	return false;
		// }

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['groupby' => 'batch_item_tasks.bit_group_note', 'where' => ' AND bit_status IN ("New", "Proses") ',  'orderby' => ' bit_id ASC ']);

		$data = [];

		if (@$batchItemTaskData) {
        	foreach ($batchItemTaskData as $index => $val ) {

        	$btn_print = '&nbsp;<a href="'.base_url().'/batch-item-task/print/'.$val->group_note.'" target="_blank"  class="btn btn-icon btn-warning" '.lang('Action.tooltip.print').'>'.lang('Action.icon.lg.print').'</a>';
        		
            $btn_task = '&nbsp;<a href="'.base_url().'/batch-item-task/'.$val->group_note.'" class="btn btn-primary btn-icon" '.lang('Action.tooltip.edit_task').' >'.lang('Action.icon.lg.edit_task').'</a>';

            $btn_request = '&nbsp;<a href="#" onclick="batchItemRequest('.$val->bi_id.')" class="btn btn-icon btn-success" data-bs-toggle="modal" data-bs-target="#kt_modal_1" '.lang('Action.tooltip.edit_request').' >'.lang('Action.icon.lg.edit_request').'</a>';

            $value['no'] = $index+1;
            $value['item_name'] = 'SKU : '.$val->item_sku.'<br>Nama : '.$val->item_name;
            $value['date'] = getTanggal($val->insert_at);
            $value['workflow_name'] = $val->workflow_name;
            $value['batch_code'] = $val->batch_code;
            $value['group_note'] = $val->group_note;
            $value['task_name'] = $val->task_name;
            $value['quantity'] = $val->quantity;
            $value['workflow_name'] = $val->workflow_name;
            $value['sku'] = $val->item_sku;
            $value['order_note'] = $val->order_note;
            $value['action'] = $btn_task.$btn_print;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function edit($id){

		if ($this->request->isAjax()) {
			$param = $this->request->getVar('param_type');
			if($param == 'repair'){
				return $this->repair($id);
			}else{
				return $this->update($id);
			}
		}

		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

		if($batchItemTaskData == false){
			echo "<script>window.close()</script>";
			exit();
		}


		$batchItemTaskCompleted = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND (bit_status="Completed" OR bit_status="Proses" ) ', 'orderby' => ' bit_id ASC ', 'groupby' => 'bit_task_id']);

		$batchItemTaskProses = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND bit_status="Proses" ','orderby' => ' bit_id ASC ']);

		$workflowModel = new WorkflowModel();

		$workflowData = $workflowModel->getWorkflowTaskGetByWorkflowRangeDown($batchItemTaskProses[0]->workflow_id, @$batchItemTaskProses[0]->order);
		
		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'data' => $batchItemTaskData,
			'data_completed' => $batchItemTaskCompleted,
			'data_task' => $workflowData,
			'batchItemID' => @$batchItemTaskData[0]->bi_id,
			'groupNote' => $id,
		];

		return view('batch-item-task/edit', $data);
	}

	public function print($id){
		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

		$biID = @$batchItemTaskData[0]->bi_id;

		$batchItemRequestModel = new BatchItemRequestModel();
		$batchItemRequestData = $batchItemRequestModel->getAll(['where' => ' AND bir_bi_id="'.$biID.'" AND bir_active=1 ', 'orderby' => ' bir_id ASC ']);

		$batchItemModel = new BatchItemModel();
		$batchItemData = $batchItemModel->getByID($biID)[0];

		$barcode = new Barcode();
		$viewBarcode = $barcode->generate($id, "SVG");

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'batch_item_task' => $batchItemTaskData,
			'batch_item' => $batchItemData,
			"batch_item_request" => $batchItemRequestData,
			'barcode' => $viewBarcode,
			'biID' => $biID
		];

		return view('batch-item-task/print', $data);
	}

	public function update($id){

		$bitID = $this->request->getPost('bit_id');
		$inputFields = $this->request->getPost('input_fields');
		$outputFields = $this->request->getPost('output_fields');
		$type = $this->request->getVar('type');

		$batchItemTaskModel = new BatchItemTaskModel();
		$bitData = $batchItemTaskModel->getAll(['where' => ' AND bit_id="'.$bitID.'" ', 'orderby' => ' bit_id ASC ']);

		$rules = [];

		if ($type == 'input') {
			$dataInputField = json_decode($bitData[0]->input_fields);
			foreach ($dataInputField as $key => $value) {
				$textRules = '';
				if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
					$textRules = 'trim|max_length[11]|numeric';
				}else{
					$textRules = 'trim|max_length[100]';
				}
				
				if ($value->is_required == 1) {
					$rules['input_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
				}else{
					$rules['input_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
				}
			}

		}else{
			$dataOutputField = json_decode($bitData[0]->output_fields);
			foreach ($dataOutputField as $key => $value) {
				$textRules = '';
				if ($value->datatype == 'Integer' || $value->datatype == 'Decimal' ) {
					$textRules = 'trim|max_length[11]|numeric';
				}else{
					$textRules = 'trim|max_length[100]';
				}
				
				if ($value->is_required == 1) {
					$rules['output_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => 'required|'.$textRules];
				}else{
					$rules['output_fields.'.$bitID.'.'.$key]	= ['label' => $value->field_name, 'rules' => $textRules];
				}
			}
		}

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
            'bit_update_admin_id' => $this->session->get('id'),
            'bit_update_at' => getCurrentTime(),
        ];

		if ($type == 'input') {
			$resultInput = [];
			$dataInputField = json_decode($bitData[0]->input_fields);

			$quantity = 0;
			$weight = 0;
			$operator = '';

			foreach ($dataInputField as $key => $value) {
				$value->answer = @$inputFields[$bitID][$key];
				$resultInput[] = $value;

				if (@$value->options == 'quantity') {
					$quantity += intval(@$inputFields[$bitID][$key]);
				}

				if (@$value->options == 'weight') {
					$weight += intval(@$inputFields[$bitID][$key]);
				}

				if (@$value->options == 'operator') {
					$operator = @$inputFields[$bitID][$key];
				}
			}

			$data['bit_input_fields'] = json_encode($resultInput);
            $data['bit_input_quantity']	= $quantity;
            $data['bit_input_weight'] = $weight;
            $data['bit_input_operator_name'] = $operator;
            $data['bit_status'] = 'Proses';
            $data['bit_status_field'] = 'output';
		}else if ($type == 'output') {

			$resultOutput = [];
			$dataOutputField = json_decode($bitData[0]->output_fields);

			$quantity = 0;
			$weight = 0;
			$operator = '';

			foreach ($dataOutputField as $key => $value) {
				$value->answer = @$outputFields[$bitID][$key];
				$resultOutput[] = $value;

				if (@$value->options == 'quantity') {
					$quantity += intval(@$outputFields[$bitID][$key]);
				}

				if (@$value->options == 'weight') {
					$weight += intval(@$outputFields[$bitID][$key]);
				}

				if (@$value->options == 'operator') {
					$operator = @$outputFields[$bitID][$key];
				}
			}

			$data['bit_output_fields'] = json_encode($resultOutput);
			$data['bit_output_quantity'] = $quantity;
            $data['bit_output_weight'] = $weight;
            $data['bit_output_operator_name'] = $operator;
            $data['bit_status'] = 'Completed';
            $data['bit_status_field'] = '';
		}

		$result = $batchItemTaskModel->updateBatchItemTask($data, $bitID);


		$orderItemModel = new OrderItemModel();

		$orderItemID = $bitData[0]->oi_id;

		$dataOi = [
			'oi_status' => 'Proses',
            'oi_insert_admin_id' => $this->session->get('id'),
            'oi_active'          => 1
		];
		
		$orderItemData = $orderItemModel->updateOrderItem($dataOi, $orderItemID);


		if ($type == 'output') {

			$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" ', 'orderby' => ' bit_id ASC ']);

			$dataBit = [];
			foreach ($batchItemTaskData as $key => $value) {
				$dataBit[] = $value->id;
			}

			$keyBit = @array_keys($dataBit, $bitID)[0];

			if(isset($dataBit[$keyBit+1])){
				$bitID2 = $dataBit[$keyBit+1];

				$result2 = $batchItemTaskModel->updateBatchItemTask([
					'bit_update_admin_id' => $this->session->get('id'),
	            	'bit_update_at' => getCurrentTime(),
	            	'bit_status' => 'Proses',
	            	'bit_status_field' => 'input',
				], $bitID2);
			}else{
				$orderItemID = $bitData[0]->oi_id;

				$dataOi = [
					'oi_status' => 'Completed',
		            'oi_insert_admin_id' => $this->session->get('id'),
		            'oi_active'          => 1
				];
				$orderItemData = $orderItemModel->updateOrderItem($dataOi, $orderItemID);
			}
		}


		$orderModel = new OrderModel();
		$orderData = $orderModel->getAll(['where' => ' AND order_active="1" AND order_status IN ("New", "Proses") ']);
		if (isset($orderData)) {
			foreach ($orderData as $key => $value) {
				$orderItemData = $orderItemModel->getByOrderID($value->id);
				$orderItemDataCompleted = $orderItemModel->getByOrderIDStatus($value->id, 'Completed');

				$orderItemDataProses = $orderItemModel->getByOrderIDStatus($value->id, 'Proses');

				$dataOrder = [
		            'order_update_admin_id' => $this->session->get('id'),
		            'order_active'          => 1
		        ];

				if (count($orderItemData) == count($orderItemDataCompleted) ) {
					$dataOrder['order_status'] = 'Completed';
				}else if((count($orderItemData) > count($orderItemDataCompleted) &&  count($orderItemDataCompleted) > 0) || (count($orderItemData) >= count($orderItemDataProses) && count($orderItemDataProses) > 0)  ){
					$dataOrder['order_status'] = 'Proses';
				}else{
					$dataOrder['order_status'] = 'New';
				}

				$orderData = $orderModel->updateOrder($dataOrder, $value->id); 
			}
		}


		$batchItemTaskData2 = $batchItemTaskModel->getAll(['where' => ' AND batches.batch_id="'.$bitData[0]->batch_id.'" AND bit_status="Proses" ', 'orderby' => ' bit_id ASC ']);

		if (empty($batchItemTaskData2)) {

			$batchModel = new BatchModel();

			$data = [
				'batch_status' 		=> 'Completed',
				'batch_update_admin_id' => $this->session->get('id'),
				'batch_update_at' 		=> getCurrentTime(),
	            'batch_active'          => 1
	        ];

			$batchID = $batchModel->updateBatches($data, $bitData[0]->batch_id);

			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'batch-item-task'
			];

			return json_encode($data);
		}


		$batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND bit_status="Proses" ', 'orderby' => ' bit_id ASC ']);
		if (empty($batchItemTaskData)) {

			$batchItemOutputModel = new BatchItemOutputModel();

			$data = [
				'bio_bi_id'=> @$bitData[0]->bi_id,
				'bio_oi_id'=> @$bitData[0]->oi_id,
				'bio_item_id'=> @$bitData[0]->item_id,
				'bio_is_melt'=> 0,
				'bio_note'=> hexdec(uniqid()).$this->session->get('id'),
				'bio_item_sku' => @$bitData[0]->item_sku,
				'bio_item_name' => @$bitData[0]->item_name,
				'bio_item_quantity' =>$quantity,
				'bio_item_weight' => $weight,
				'bio_insert_at' => getCurrentTime(),
				'bio_insert_admin_id' => $this->session->get('id')
			];
		
			$result = $batchItemOutputModel->addBatchItemOutput($data);

			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'batch-item-task'
			];

			return json_encode($data);
		}

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'reload' => true
			];

			return json_encode($data);
		}
	}


	public function report(){
		if($this->request->isAjax()){
            return $this->dataReport();
        }

        if ($this->request->getGet('export-excel') == true) {
            return $this->exportReport();
        }

        $data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task-report',
		];

		return view('batch-item-task/report', $data);
	}

	public function dataReport(){
		if (!$this->request->isAjax()) {
			return false;
		}

		$note = $this->request->getVar('note', FILTER_SANITIZE_STRING);
		$batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
		$sku = $this->request->getVar('sku', FILTER_SANITIZE_STRING);
		$fromDateNote = $this->request->getVar('from_date_note', FILTER_SANITIZE_STRING);
 		$toDateNote = $this->request->getVar('to_date_note', FILTER_SANITIZE_STRING);
	 	$fromDateStart = $this->request->getVar('from_date_start', FILTER_SANITIZE_STRING);
		$toDateStart = $this->request->getVar('to_date_start', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDateNote != '') {
            $queryWhere .= ' AND order_date >="'.$fromDateNote.'" ';
        }

        if ($toDateNote != '') {
            $queryWhere .= ' AND order_date <= "'.$toDateNote.'" ';
        }

        if ($fromDateStart != '') {
            $queryWhere .= ' AND batch_item_tasks.bit_insert_at >="'.$fromDateStart.'" ';
        }

        if ($toDateStart != '') {
            $queryWhere .= ' AND batch_item_tasks.bit_insert_at <= "'.$toDateStart.'" ';
        }

        if ($sku != '') {
            $queryWhere .= ' AND item_sku = "'.$sku.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($note != '') {
            $queryWhere .= ' AND order_note = "'.$note.'" ';
        }
		
		$batchItemTaskModel = new BatchItemTaskModel();

		$batchItemTaskData = $batchItemTaskModel->getAll(['where'=> $queryWhere]);

		$data = [];

		if (@$batchItemTaskData) {
        	foreach ($batchItemTaskData as $index => $val ) {

	            $value['note'] = $val->order_note;
	            $value['batch_code'] = $val->batch_code;
	            $value['item_sku'] = $val->item_sku;
	            $value['item_name'] = $val->item_name;
	            $value['task_name'] = $val->task_name;
	            $value['status'] = $val->status;
	            $value['input_quantity'] = $val->input_quantity;
	            $value['output_quantity'] = $val->output_quantity;
	            $value['input_weight'] = $val->input_weight;
	            $value['output_weight'] = $val->output_weight;
	            $value['start_date'] = $val->insert_at.'<br>'.$val->input_operator_name;
	            if ($val->status == 'Completed') {
	            	$value['end_date'] = $val->update_at.'<br>'.$val->output_operator_name;
	            }else{
	            	$value['end_date'] = '';
	            }

	            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
	}

	public function exportReport(){
		$note = $this->request->getVar('note', FILTER_SANITIZE_STRING);
		$batchCode = $this->request->getVar('batch_code', FILTER_SANITIZE_STRING);
		$sku = $this->request->getVar('sku', FILTER_SANITIZE_STRING);
		$fromDateNote = $this->request->getVar('from_date_note', FILTER_SANITIZE_STRING);
 		$toDateNote = $this->request->getVar('to_date_note', FILTER_SANITIZE_STRING);
	 	$fromDateStart = $this->request->getVar('from_date_start', FILTER_SANITIZE_STRING);
		$toDateStart = $this->request->getVar('to_date_start', FILTER_SANITIZE_STRING);

        $queryWhere = '';

        if ($fromDateNote != '') {
            $queryWhere .= ' AND order_date >="'.$fromDateNote.'" ';
        }

        if ($toDateNote != '') {
            $queryWhere .= ' AND order_date <= "'.$toDateNote.'" ';
        }

        if ($fromDateStart != '') {
            $queryWhere .= ' AND batch_item_tasks.bit_insert_at >="'.$fromDateStart.'" ';
        }

        if ($toDateStart != '') {
            $queryWhere .= ' AND batch_item_tasks.bit_insert_at <= "'.$toDateStart.'" ';
        }

        if ($sku != '') {
            $queryWhere .= ' AND item_sku = "'.$sku.'" ';
        }

        if ($batchCode != '') {
            $queryWhere .= ' AND batch_code = "'.$batchCode.'" ';
        }

        if ($note != '') {
            $queryWhere .= ' AND order_note = "'.$note.'" ';
        }
		
		$batchItemTaskModel = new BatchItemTaskModel();

		$batchItemTaskData = $batchItemTaskModel->getAll(['where'=> $queryWhere]);

		$data = [
			'title' => lang('Menu.order.batch-item-task.main'),
			'menu' => 'batch-item-task',
			'data' => $batchItemTaskData,
		];

		return view('batch-item-task/excel', $data);

	}

	public function repair($id){

		$batchItemTaskModel = new BatchItemTaskModel();
		$taskModel = new TaskModel();

		$taskOrder = $this->request->getVar("task_order");
		$weight = $this->request->getVar("weight");
		$quantity = $this->request->getVar("quantity");

		$rules = [
            'task_order' => ['label' => 'Order Task', 'rules' => 'required|max_length[11]'],
            'weight' => ['label' => 'Berat', 'rules' => 'required|max_length[11]'],
            'quantity' => ['label' => 'Jumlah', 'rules' => 'required|max_length[11]'],
        ];

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$workflowModel = new WorkflowModel();

		// $batchItemTaskData = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND bit_id="'.$bitID.'" ', 'orderby' => ' bit_id ASC ', 'groupby' => 'bit_task_id']);

		$batchItemTaskProses = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND bit_status="Proses" ','orderby' => ' bit_id ASC ']);
		$bitID = @$batchItemTaskProses[0]->id;
		$biID = @$batchItemTaskProses[0]->bi_id;
		$workflowID = $batchItemTaskProses[0]->workflow_id;

		$workflowTaskData = $workflowModel->getWorkflowTaskGetByWorkflowRangeUp($workflowID, $taskOrder);

		$batchItemTaskCompleted = $batchItemTaskModel->getAll(['where' => ' AND bit_group_note="'.$id.'" AND (bit_status="Completed" OR bit_status="Proses" ) ', 'orderby' => ' bit_id ASC ']);
		// $data2 = [];
		// $data2['bit_status'] = 'Repaired';
		// $data2['bit_is_repaired'] = 1;
		// $data2['bit_update_at'] = getCurrentTime();
		// $data2['bit_update_admin_id'] = $this->session->get('id');
		// $result = $batchItemTaskModel->updateBatchItemTask($data2, $bitID);
		
		$groupNote = $this->groupNote();

		$data_ = [];
		foreach ($batchItemTaskCompleted as $key_ => $value_) {
			$data_[] = [
				'bit_bi_id' => $value_->bi_id,
				'bit_order' => $value_->order,
				'bit_is_repaired' => $value_->is_repaired,
				'bit_repaired_bit_id' => $bitID,
				'bit_workflow_id' => $value_->workflow_id,
				'bit_task_id' => $value_->task_id,
				'bit_group_note' => $groupNote,
				'bit_input_operator_name' => $value_->input_operator_name,
				'bit_output_operator_name' => $value_->output_operator_name,
				'bit_input_fields' => $value_->input_fields,
				'bit_output_fields' => $value_->output_fields,
				'bit_input_quantity' => $value_->input_quantity,
				'bit_output_quantity' => $value_->output_quantity,
				'bit_input_weight' => $value_->input_weight,
				'bit_output_weight' => $value_->output_weight,
				'bit_status_field' => $value_->status_field,
				'bit_status' => 'Completed',
				'bit_insert_at' => $value_->insert_at,
				'bit_insert_admin_id' => $value_->insert_admin_id
			];
		}

		$result = $batchItemTaskModel->addBatchItemTaskBulk($data_);


		$data = [];
		foreach ($workflowTaskData as $key => $value) {

			$taskInputData = [];
			$taskResult = $taskModel->taskInputFieldGetByTask($value->task_id, 'input');
			foreach ($taskResult as $key2 => $value2) {
				$taskInputData[$key2]['field_name'] = $value2->name;
				$taskInputData[$key2]['is_required'] = $value2->is_required;
				$taskInputData[$key2]['datatype'] = $value2->datatype;
				$taskInputData[$key2]['options'] = $value2->options;
				$taskInputData[$key2]['answer'] = '';
				if($value2->options == 'quantity'){
					$taskInputData[$key2]['answer'] = $quantity;
				}else if($value2->options == 'weight'){
					$taskInputData[$key2]['answer'] = $weight;
				}
			}

			$taskOutputData = [];
			$taskResult3 = $taskModel->taskInputFieldGetByTask($value->task_id, 'output');
			foreach ($taskResult3 as $key3 => $value3) {
				$taskOutputData[$key3]['field_name'] = $value3->name;
				$taskOutputData[$key3]['is_required'] = $value3->is_required;
				$taskOutputData[$key3]['datatype'] = $value3->datatype;
				$taskOutputData[$key3]['options'] = $value3->options;
				$taskOutputData[$key3]['answer'] = '';
				if($value2->options == 'quantity'){
					$taskInputData[$key3]['answer'] = $quantity;
				}else if($value2->options == 'weight'){
					$taskInputData[$key3]['answer'] = $weight;
				}
			}

			$data[] = [
				'bit_bi_id' => $biID,
				'bit_order' => $value->order,
				'bit_is_repaired' => ($key == 0)? 1 : 0,
				'bit_repaired_bit_id' => $bitID,
				'bit_workflow_id' => $value->workflow_id,
				'bit_task_id' => $value->task_id,
				'bit_group_note' => $groupNote,
				'bit_input_operator_name' => '',
				'bit_output_operator_name' => '',
				'bit_input_fields' => json_encode($taskInputData),
				'bit_output_fields' => json_encode($taskOutputData),
				'bit_input_quantity' => ($key == 0)? $weight : 0,
				'bit_output_quantity' => ($key == 0)? $quantity : 0,
				'bit_input_weight' => 0,
				'bit_output_weight' => 0,
				'bit_status_field' => ($key == 0)? 'input' : '',
				'bit_status' => ($key == 0)? 'Proses' : 'New',
				'bit_insert_at' => getCurrentTime(),
				'bit_insert_admin_id' => $this->session->get('id')
			];	
		}

		$result = $batchItemTaskModel->addBatchItemTaskBulk($data);

		if ($result) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'batch-item-task/'.$groupNote,
				'target' => '_blank'
			];

			return json_encode($data);
		}

	}

	public function groupNote(){
		$batchItemTaskModel = new BatchItemTaskModel();
		$batchItemTaskData = $batchItemTaskModel->getAll(['select'=> 'COUNT(*) as "count_bit" ', 'groupby' => 'batch_item_tasks.bit_group_note', 'where' => ' AND bit_status IN ("New", "Proses") ',  'orderby' => ' bit_id ASC ']);
		if($batchItemTaskData){
			$number = intval($batchItemTaskData[0]->count_bit)+1;
		}else{
			$number = 1;
		}

		return randomString($length = 8).'-'.str_pad($number, 6, "0", STR_PAD_LEFT);
	}

}