<?php

namespace App\Controllers;

use App\Models\GoldContentModel;
use App\Models\ItemModel;

class GoldContent extends BaseController
{
	public function getAll()
	{

		if($this->request->isAjax()){
			return $this->data();
		}

		$data = [
			'title' => lang('Menu.master.gold-content.main'),
			'menu' => 'gold-content',
		];

		return view('gold-content/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$goldContentModel = new GoldContentModel();

		$goldContentData = $goldContentModel->getAll(); 

		$data = [];

		if (@$goldContentData) {
        	foreach ($goldContentData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/gold-content/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/gold-content/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['multiplier'] = $val->multiplier;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addGoldContent()
	{
		if (!$this->request->isAjax()) {
			$data = [
				'title' => lang('Menu.master.gold-content.main'),
				'menu' => 'gold-content',
			];

			return view('gold-content/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Kadar Emas', 'rules' => 'required|max_length[255]'],
            'multiplier'             => ['label' => 'Pengali/Multiplier', 'rules' => 'required|numeric|max_length[255]|' ],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$data = [
			'gc_code'		     => url_title($this->request->getPost('name'), '-', TRUE),
			'gc_name' 		     => $this->request->getPost('name'),
			'gc_multiplier' 	 => $this->request->getPost('multiplier'),
            'gc_insert_admin_id' => $this->session->get('id'),
            'gc_active'          => 1
        ];

		$goldContentModel = new GoldContentModel();

		$goldContentData = $goldContentModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($goldContentData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }

		$goldContentID = $goldContentModel->addGoldContent($data);

		if ($goldContentID) {
			$data = [
				'redirect' => 'gold-content',
				'success'=> lang('Validation.success_add'),
			];

			return json_encode($data);
		}

	}

	public function updateGoldContent($id)
	{
		$goldContentModel = new GoldContentModel();
		$goldContentData = $goldContentModel->getByID($id);

		if (!$this->request->isAjax()) {

			$data = [
				'data' => (isset($goldContentData))? $goldContentData : [],
				'title' => lang('Menu.master.gold-content.main'),
				'menu' => 'gold-content',
			];

			return view('gold-content/edit',  $data);
		}

		$rules = [
            'multiplier'             => ['label' => 'Pengali/Multiplier', 'rules' => 'required|numeric|max_length[255]|' ],
        ];

        $data = [
			'gc_code'		     => url_title($this->request->getPost('name')),
			'gc_name' 		     => $this->request->getPost('name'),
			'gc_multiplier' 	 => $this->request->getPost('multiplier'),
            'gc_update_admin_id' => $this->session->get('id'),
            'gc_active'          => 1
        ];

        $data2 = [
			'gch_gc_id'		     => $id,
			'gch_multiplier_from'=> $goldContentData->multiplier,
			'gch_multiplier_to' => $this->request->getPost('multiplier'),
            'gch_update_admin_id' => $this->session->get('id'),
        ];

        if ($goldContentData->name != $data['gc_name']) {
        	$goldContentDataByCode = $goldContentModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($goldContentDataByCode) > 0) {
					$msg = [
		                'error' => lang('Validation.name_already_exist'),
		            ];
		            return json_encode($msg);
	        }
        }

		if (!$this->validate($rules)) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$goldContentModel = new GoldContentModel();

		$goldContentData = $goldContentModel->updateGoldContent($data, $id); 

		$goldContentHistoryID = $goldContentModel->addGoldContentHistory($data2);

		if ($goldContentData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'gold-content'
			];

			return json_encode($data);
		}
	}

	public function deleteGoldContent($id)
	{

		$itemModel = new ItemModel();
		if(count($itemModel->getAll(["where" => ' AND item_gc_id="'.$id.'" AND item_active=1'])) > 0 ){
			$data = [
				'error'=> lang('Validation.gc_already_use'),
			];
			return json_encode($data);
		}

		$goldContentModel = new GoldContentModel();

		$goldContentData = $goldContentModel->deleteGoldContent($id, $this->session->get('id'));

		if ($goldContentData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}