<?php

namespace App\Controllers;

use App\Models\TaskModel;
use App\Models\AdminModel;

class Task extends BaseController
{
	public function getAll()
	{
		if($this->request->isAjax()){
			return $this->data();
		}
		
		$data = [
			'title' => lang('Menu.master.task.main'),
			'menu' => 'task',
		];

		return view('task/index', $data);
	}

	public function data()
	{
		if (!$this->request->isAjax()) {
			return false;
		}
		
		$taskModel = new TaskModel();

		$taskData = $taskModel->getAll(); 

		$data = [];

		if (@$taskData) {
        	foreach ($taskData as $index => $val ) {
        		
            $btn_delete = '&nbsp;<a href="javascript:;" class="btn btn-icon btn-danger" onclick="return deleteData(`'.base_url().'/task/delete/'.$val->id.'`)" '.lang('Action.tooltip.delete').'>'.lang('Action.icon.lg.delete').'</a>';

            $btn_edit = '&nbsp;<a href="'.base_url().'/task/edit/'.$val->id.'" class="btn btn-success btn-icon" '.lang('Action.tooltip.edit').'>'.lang('Action.icon.lg.edit').'</a>';

            $value['no'] = $index+1;
            $value['code'] = $val->code;
            $value['name'] = $val->name;
            $value['insert_at'] = $val->insert_at;
            $value['action'] = $btn_edit.$btn_delete;

            array_push($data,$value);
        	}
    	}

        $search = $this->request->getVar('search');

        if (isset($search['value']) && $search['value']) {
            $data = $this->arraySearch($data, $search['value']);
        }

        $totalRecords = $totalDisplay = count($data);

        $order = $this->request->getVar('order');

        if (isset($order[0]['column']) && $order[0]['dir']) {
            $column = $order[0]['column'];
            $dir    = $order[0]['dir'];
            usort($data, function ($a, $b) use ($column, $dir) {
                $a = array_slice($a, $column, 1);
                $b = array_slice($b, $column, 1);
                $a = array_pop($a);
                $b = array_pop($b);

                if ($dir === 'asc') {
                    return $a > $b ? 1 : -1;
                }

                return $a < $b ? 1 : -1;
            });
        }

        // pagination length
        if ($this->request->getVar('length')) {
            $data = array_splice($data, $this->request->getVar('start'), $this->request->getVar('length'));
        }

        $result = [
            'recordsTotal'    => $totalRecords,
            'recordsFiltered' => $totalDisplay,
            'data'            => $data,
        ];

        echo json_encode($result, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
    }

	public function addTask()
	{
		$taskModel = new TaskModel();
		if (!$this->request->isAjax()) {

			$adminModel = new AdminModel();

			$data = [
				'title' => lang('Menu.master.task.main'),
				'menu' => 'task',
				'input_field_input' => $taskModel->inputFieldGetAll(['type' => 'Input']),
				'input_field_output' => $taskModel->inputFieldGetAll(['type' => 'Output']),
				'data_admin' => $adminModel->getAll()
			];

			return view('task/create', $data);
		}

		if (!$this->validate([
            'name'             => ['label' => 'Nama Divisi', 'rules' => 'required|max_length[255]'],
            'input.0'             => ['label' => 'List Input', 'rules' => 'required'],
            'output.0'             => ['label' => 'List Output', 'rules' => 'required'],
            'notification_input.0' => ['label' => 'List Notifikasi Input', 'rules' => 'required'],
            'notification_output.0' => ['label' => 'List Notifikasi Output', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$input = $this->request->getPost('input');
		$inputRequired = $this->request->getPost('input_required');

		$output = $this->request->getPost('output');
		$outputRequired = $this->request->getPost('output_required');

		$notificationInput = $this->request->getPost('notification_input');
		$notificationOutput = $this->request->getPost('notification_output');

		$data = [
			'task_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'task_name' 		     => $this->request->getPost('name'),
			'task_input_notification_list' => ($notificationInput != '')? implode(',', $notificationInput ) : '',
			'task_output_notification_list' => ($notificationOutput != '')? implode(',', $notificationOutput ) : '',
            'task_insert_admin_id' => $this->session->get('id'),
            'task_active'          => 1
        ];


        $taskData = $taskModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
        if (count($taskData) > 0) {
				$msg = [
	                'error' => lang('Validation.name_already_exist'),
	            ];
	            return json_encode($msg);
        }

		$taskData = $taskModel->addTask($data); 

		$data2 = [];
		foreach ($input as $key => $value) {
			$data2[] = [
				'tif_task_id' => $taskData,
				'tif_if_id' => $value,
				'tif_order' => $key+1,
				'tif_is_required' => (@$inputRequired[$value] == 1)? 1 : 0,
				'tif_insert_admin_id' => $this->session->get('id'),
				'tif_active' => 1,
				'tif_type' => 'input',
			];
		}

		foreach ($output as $key => $value) {
			$data2[] = [
				'tif_task_id' => $taskData,
				'tif_if_id' => $value,
				'tif_order' => $key+1,
				'tif_is_required' => (@$outputRequired[$value] == 1)? 1 : 0,
				'tif_insert_admin_id' => $this->session->get('id'),
				'tif_active' => 1,
				'tif_type' => 'output',
			];
		}
		
		$taskInputFieldData = $taskModel->addTaskInputFieldBulk($data2);

		if ($taskInputFieldData) {
			$data = [
				'success'=> lang('Validation.success_add'),
				'redirect' => 'task'
			];

			return json_encode($data);
		}

	}

	public function updateTask($id)
	{
		$taskModel = new TaskModel();

		$taskData = $taskModel->getByID($id);

		if (!$this->request->isAjax()) {

			$adminModel = new AdminModel();

			$dataInputInputField = $taskModel->taskInputFieldGetByTask($id, 'input');

			$dataOutputInputField = $taskModel->taskInputFieldGetByTask($id, 'output');

			$data = [
				'title' => lang('Menu.master.task.main'),
				'menu' => 'task',
				'data' => (isset($taskData))? $taskData : [],
				'input_input_field' => $dataInputInputField,
				'output_input_field' => $dataOutputInputField,
				'input_field' => $taskModel->inputFieldGetAll(),
				'data_admin' => $adminModel->getAll()
			];

			return view('task/edit',  $data);
		}


		if (!$this->validate([
            'name'             => ['label' => 'Nama Divisi', 'rules' => 'required|max_length[255]'],
            'input.0'             => ['label' => 'List Input', 'rules' => 'required'],
            'output.0'             => ['label' => 'List Output', 'rules' => 'required'],
            'notification_input.0' => ['label' => 'List Notifikasi Input', 'rules' => 'required'],
            'notification_output.0' => ['label' => 'List Notifikasi Output', 'rules' => 'required'],
        ])) {
			$msg = [
                'error' => $this->validation->listErrors(),
            ];
            return json_encode($msg);
		}

		$tifIDInput = $this->request->getPost('tif_id_input');
		$tifIDOutput = $this->request->getPost('tif_id_output');

		$input = $this->request->getPost('input');
		$inputRequired = $this->request->getPost('input_required');

		$output = $this->request->getPost('output');
		$outputRequired = $this->request->getPost('output_required');

		$notificationInput = $this->request->getPost('notification_input');
		$notificationOutput = $this->request->getPost('notification_output');

		$data = [
			'task_code'		     => url_title($this->request->getPost('name'), "-", TRUE),
			'task_name' 		     => $this->request->getPost('name'),
			'task_input_notification_list' => ($notificationInput != '')? implode(',', $notificationInput ) : '',
			'task_output_notification_list' => ($notificationOutput != '')? implode(',', $notificationOutput ) : '',
            'task_update_admin_id' => $this->session->get('id'),
            'task_active'          => 1
        ];

        if ($taskData->name != $data['task_name']) {
        	$taskDataByCode = $taskModel->getByCode(url_title($this->request->getPost('name'), "-", TRUE));
	        if (count($taskDataByCode) > 0) {
					$msg = [
		                'error' => lang('Validation.name_already_exist'),
		            ];
		            return json_encode($msg);
	        }
        }

		$taskData = $taskModel->updateTask($data, $id); 

		$data2 = [];
		foreach ($input as $key => $value) {
			$data2[] = [
				'tif_task_id' => $id,
				'tif_if_id' => $value,
				'tif_order' => $key+1,
				'tif_is_required' => (@$inputRequired[$value] == 1)? 1 : 0,
				'tif_type' => 'input',
				'tif_insert_admin_id' => $this->session->get('id'),
				'tif_active' => 1,
				'tif_type' => 'input',
			];
		}

		foreach ($output as $key => $value) {
			$data2[] = [
				'tif_task_id' => $id,
				'tif_if_id' => $value,
				'tif_order' => $key+1,
				'tif_is_required' => (@$outputRequired[$value] == 1)? 1 : 0,
				'tif_insert_admin_id' => $this->session->get('id'),
				'tif_active' => 1,
				'tif_type' => 'output',
			];
		}

		$taskModel->deleteTaskInputField($id, $this->session->get('id'));
		$taskInputFieldData = $taskModel->addTaskInputFieldBulk($data2);

		if ($taskInputFieldData) {
			$data = [
				'success'=> lang('Validation.success_edit'),
				'redirect' => 'task'
			];

			return json_encode($data);
		}

	}

	public function deleteTask($id)
	{
		
		$taskModel = new TaskModel();

		$taskData = $taskModel->deleteTask($id, $this->session->get('id')); 

		if ($taskData) {
			$data = [
				'success'=> lang('Validation.success_delete'),
			];

			return json_encode($data);
		}
	}

}